@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,14 +19,13 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BGP_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
 #include "bgp.h"
 #include "bgp_xcs.h"
+#include "rpki/rpki.h"
+#include "bgp_blackhole.h"
 #include "thread_pool.h"
 #if defined WITH_RABBITMQ
 #include "amqp_common.h"
@@ -34,15 +33,44 @@
 #ifdef WITH_KAFKA
 #include "kafka_common.h"
 #endif
+#if defined WITH_ZMQ
+#include "zmq_common.h"
+#endif
+#if defined WITH_AVRO
+#include "plugin_cmn_avro.h"
+#endif
+#include "bgp_lg.h"
 
-/* variables to be exported away */
+/* Global variables */
 thread_pool_t *bgp_pool;
+struct bgp_peer *peers;
+struct bgp_peer_cache_bucket *peers_cache, *peers_port_cache;
+char *std_comm_patterns[MAX_BGP_COMM_PATTERNS];
+char *ext_comm_patterns[MAX_BGP_COMM_PATTERNS];
+char *lrg_comm_patterns[MAX_BGP_COMM_PATTERNS];
+char *std_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
+char *lrg_comm_patterns_to_asn[MAX_BGP_COMM_PATTERNS];
+struct bgp_comm_range peer_src_as_ifrange; 
+struct bgp_comm_range peer_src_as_asrange; 
+u_int32_t (*bgp_route_info_modulo)(struct bgp_peer *, path_id_t *, int);
+struct bgp_rt_structs inter_domain_routing_dbs[FUNC_TYPE_MAX], *bgp_routing_db;
+struct bgp_misc_structs inter_domain_misc_dbs[FUNC_TYPE_MAX], *bgp_misc_db;
+struct bgp_xconnects bgp_xcs_map;
 
 /* Functions */
-void nfacctd_bgp_wrapper()
+void bgp_daemon_wrapper()
 {
   /* initialize variables */
-  if (!config.nfacctd_bgp_port) config.nfacctd_bgp_port = BGP_TCP_PORT;
+  if (!config.bgp_daemon_port) config.bgp_daemon_port = BGP_TCP_PORT;
+
+#if defined WITH_ZMQ
+  if (config.bgp_lg) bgp_lg_wrapper();
+#else
+  if (config.bgp_lg) {
+    Log(LOG_ERR, "ERROR ( %s/core/lg ): 'bgp_daemon_lg' requires --enable-zmq. Exiting.\n", config.name);
+    exit_gracefully(1);
+  }
+#endif
 
   /* initialize threads pool */
   bgp_pool = allocate_thread_pool(1);
@@ -61,26 +89,27 @@ void skinny_bgp_daemon()
 
 void skinny_bgp_daemon_online()
 {
-  int slen, ret, rc, peers_idx, allowed;
+  int ret, rc, peers_idx, allowed, yes=1;
+#if (defined IPV6_BINDV6ONLY)
+  int no=0;
+#endif
   int peers_idx_rr = 0, peers_xconnect_idx_rr = 0, max_peers_idx = 0;
   struct plugin_requests req;
   struct host_addr addr;
   struct bgp_peer *peer;
   char bgp_reply_pkt[BGP_BUFFER_SIZE], *bgp_reply_pkt_ptr;
   char bgp_peer_str[INET6_ADDRSTRLEN], bgp_xconnect_peer_str[BGP_XCONNECT_STRLEN];
-#if defined ENABLE_IPV6
   struct sockaddr_storage server, client;
-#else
-  struct sockaddr server, client;
-#endif
   afi_t afi;
   safi_t safi;
-  int clen = sizeof(client), yes=1, no=0;
-  time_t now, dump_refresh_deadline;
+  time_t now, dump_refresh_deadline = {0};
   struct hosts_table allow;
   struct bgp_md5_table bgp_md5;
   struct timeval dump_refresh_timeout, *drt_ptr;
   struct bgp_peer_batch bp_batch;
+  socklen_t slen, clen = sizeof(client);
+
+  sigset_t signal_set;
 
   /* select() stuff */
   fd_set read_descs, bkp_read_descs; 
@@ -101,65 +130,49 @@ void skinny_bgp_daemon_online()
   bgp_attr_init(config.bgp_table_attr_hash_buckets, bgp_routing_db);
 
   /* socket creation for BGP server: IPv4 only */
-#if (defined ENABLE_IPV6)
-  if (!config.nfacctd_bgp_ip) {
+  if (!config.bgp_daemon_ip) {
     struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *)&server;
 
     sa6->sin6_family = AF_INET6;
-    sa6->sin6_port = htons(config.nfacctd_bgp_port);
+    sa6->sin6_port = htons(config.bgp_daemon_port);
     slen = sizeof(struct sockaddr_in6);
   }
-#else
-  if (!config.nfacctd_bgp_ip) {
-    struct sockaddr_in *sa4 = (struct sockaddr_in *)&server;
-
-    sa4->sin_family = AF_INET;
-    sa4->sin_addr.s_addr = htonl(0);
-    sa4->sin_port = htons(config.nfacctd_bgp_port);
-    slen = sizeof(struct sockaddr_in);
-  }
-#endif
   else {
-    trim_spaces(config.nfacctd_bgp_ip);
-    ret = str_to_addr(config.nfacctd_bgp_ip, &addr);
+    trim_spaces(config.bgp_daemon_ip);
+    ret = str_to_addr(config.bgp_daemon_ip, &addr);
     if (!ret) {
       Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_daemon_ip' value is not a valid IPv4/IPv6 address. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
-    slen = addr_to_sa((struct sockaddr *)&server, &addr, config.nfacctd_bgp_port);
+    slen = addr_to_sa((struct sockaddr *)&server, &addr, config.bgp_daemon_port);
   }
 
-  if (!config.nfacctd_bgp_max_peers) config.nfacctd_bgp_max_peers = MAX_BGP_PEERS_DEFAULT;
-  Log(LOG_INFO, "INFO ( %s/%s ): maximum BGP peers allowed: %d\n", config.name, bgp_misc_db->log_str, config.nfacctd_bgp_max_peers);
+  if (!config.bgp_daemon_max_peers) config.bgp_daemon_max_peers = MAX_BGP_PEERS_DEFAULT;
+  Log(LOG_INFO, "INFO ( %s/%s ): maximum BGP peers allowed: %d\n", config.name, bgp_misc_db->log_str, config.bgp_daemon_max_peers);
 
-  peers = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer));
+  peers = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer));
   if (!peers) {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers structure. Terminating thread.\n", config.name, bgp_misc_db->log_str);
     exit_gracefully(1);
   }
-  memset(peers, 0, config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer));
+  memset(peers, 0, config.bgp_daemon_max_peers*sizeof(struct bgp_peer));
 
   if (config.bgp_lg) {
-    if (config.acct_type != ACCT_PMBGP) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): bgp_daemon_lg feature not supported for this daemon. Exiting ...\n", config.name, config.type);
-      exit_gracefully(1);
-    }
-
-    peers_cache = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_cache_bucket));
+    peers_cache = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer_cache_bucket));
     if (!peers_cache) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers cache structure. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
 
-    bgp_peer_cache_init(peers_cache, config.nfacctd_bgp_max_peers);
+    bgp_peer_cache_init(peers_cache, config.bgp_daemon_max_peers);
 
-    peers_port_cache = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_cache_bucket));
+    peers_port_cache = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer_cache_bucket));
     if (!peers_port_cache) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers cache structure (2). Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
 
-    bgp_peer_cache_init(peers_port_cache, config.nfacctd_bgp_max_peers);
+    bgp_peer_cache_init(peers_port_cache, config.bgp_daemon_max_peers);
   }
   else {
     peers_cache = NULL;
@@ -178,12 +191,12 @@ void skinny_bgp_daemon_online()
     memset(&req, 0, sizeof(req));
 
     /* Setting up the pool */
-    bgp_xcs_map.pool = malloc((config.nfacctd_bgp_max_peers + 1) * sizeof(struct bgp_xconnect));
+    bgp_xcs_map.pool = malloc((config.bgp_daemon_max_peers + 1) * sizeof(struct bgp_xconnect));
     if (!bgp_xcs_map.pool) {
       Log(LOG_ERR, "ERROR ( %s/%s ): unable to allocate BGP xconnect pool. Exiting ...\n", config.name, config.type);
       exit_gracefully(1);
     }
-    else memset(bgp_xcs_map.pool, 0, (config.nfacctd_bgp_max_peers + 1) * sizeof(struct bgp_xconnect));
+    else memset(bgp_xcs_map.pool, 0, (config.bgp_daemon_max_peers + 1) * sizeof(struct bgp_xconnect));
 
     req.key_value_table = (void *) &bgp_xcs_map;
     load_id_file(MAP_BGP_XCS, config.bgp_xconnect_map, NULL, &req, &bgp_xcs_allocated);
@@ -193,25 +206,51 @@ void skinny_bgp_daemon_online()
     bgp_xcs_map.num = 0;
   }
 
-  if (config.nfacctd_bgp_msglog_file || config.nfacctd_bgp_msglog_amqp_routing_key || config.nfacctd_bgp_msglog_kafka_topic) {
-    if (config.nfacctd_bgp_msglog_file) bgp_misc_db->msglog_backend_methods++;
-    if (config.nfacctd_bgp_msglog_amqp_routing_key) bgp_misc_db->msglog_backend_methods++;
-    if (config.nfacctd_bgp_msglog_kafka_topic) bgp_misc_db->msglog_backend_methods++;
+  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+    rpki_daemon_wrapper();
+
+    /* Let's give the RPKI thread some advantage to create its structures */
+    sleep(DEFAULT_SLOTH_SLEEP_TIME);
+  }
+
+  if (config.bgp_blackhole_stdcomm_list) {
+#if defined WITH_ZMQ
+    struct p_zmq_host *bgp_blackhole_zmq_host = NULL;
+    char inproc_blackhole_str[] = "inproc://bgp_blackhole";
+    (void)inproc_blackhole_str;
+
+    bgp_blackhole_daemon_wrapper();
+
+    /* Let's give the BGP blackhole thread some advantage to create its structures */
+    sleep(DEFAULT_SLOTH_SLEEP_TIME);
+
+    bgp_blackhole_zmq_host = bgp_blackhole_misc_db->bgp_blackhole_zmq_host;
+    p_zmq_push_connect_setup(bgp_blackhole_zmq_host);
+#else
+    Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_blackhole_stdcomm_list' requires compiling with --enable-zmq. Exiting ..\n", config.name, bgp_misc_db->log_str);
+    exit_gracefully(1);
+#endif
+  }
+
+  if (config.bgp_daemon_msglog_file || config.bgp_daemon_msglog_amqp_routing_key || config.bgp_daemon_msglog_kafka_topic) {
+    if (config.bgp_daemon_msglog_file) bgp_misc_db->msglog_backend_methods++;
+    if (config.bgp_daemon_msglog_amqp_routing_key) bgp_misc_db->msglog_backend_methods++;
+    if (config.bgp_daemon_msglog_kafka_topic) bgp_misc_db->msglog_backend_methods++;
 
     if (bgp_misc_db->msglog_backend_methods > 1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): bgp_daemon_msglog_file, bgp_daemon_msglog_amqp_routing_key and bgp_daemon_msglog_kafka_topic are mutually exclusive. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
 
-    bgp_misc_db->peers_log = malloc(config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_log));
+    bgp_misc_db->peers_log = malloc(config.bgp_daemon_max_peers*sizeof(struct bgp_peer_log));
     if (!bgp_misc_db->peers_log) {
       Log(LOG_ERR, "ERROR ( %s/%s ): Unable to malloc() BGP peers log structure. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
-    memset(bgp_misc_db->peers_log, 0, config.nfacctd_bgp_max_peers*sizeof(struct bgp_peer_log));
+    memset(bgp_misc_db->peers_log, 0, config.bgp_daemon_max_peers*sizeof(struct bgp_peer_log));
     bgp_peer_log_seq_init(&bgp_misc_db->log_seq);
 
-    if (config.nfacctd_bgp_msglog_amqp_routing_key) {
+    if (config.bgp_daemon_msglog_amqp_routing_key) {
 #ifdef WITH_RABBITMQ
       bgp_daemon_msglog_init_amqp_host();
       p_amqp_connect_to_publish(&bgp_daemon_msglog_amqp_host);
@@ -220,7 +259,7 @@ void skinny_bgp_daemon_online()
 #endif
     }
 
-    if (config.nfacctd_bgp_msglog_kafka_topic) {
+    if (config.bgp_daemon_msglog_kafka_topic) {
 #ifdef WITH_KAFKA
       bgp_daemon_msglog_init_kafka_host();
 #else
@@ -257,56 +296,54 @@ void skinny_bgp_daemon_online()
 
   config.bgp_sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_STREAM, 0);
   if (config.bgp_sock < 0) {
-#if (defined ENABLE_IPV6)
     /* retry with IPv4 */
-    if (!config.nfacctd_bgp_ip) {
+    if (!config.bgp_daemon_ip) {
       struct sockaddr_in *sa4 = (struct sockaddr_in *)&server;
 
       sa4->sin_family = AF_INET;
       sa4->sin_addr.s_addr = htonl(0);
-      sa4->sin_port = htons(config.nfacctd_bgp_port);
+      sa4->sin_port = htons(config.bgp_daemon_port);
       slen = sizeof(struct sockaddr_in);
 
       config.bgp_sock = socket(((struct sockaddr *)&server)->sa_family, SOCK_STREAM, 0);
     }
-#endif
 
     if (config.bgp_sock < 0) {
       Log(LOG_ERR, "ERROR ( %s/%s ): thread socket() failed. Terminating thread.\n", config.name, bgp_misc_db->log_str);
       exit_gracefully(1);
     }
   }
-  if (config.nfacctd_bgp_ipprec) {
-    int opt = config.nfacctd_bgp_ipprec << 5;
+  if (config.bgp_daemon_ipprec) {
+    int opt = config.bgp_daemon_ipprec << 5;
 
-    rc = setsockopt(config.bgp_sock, IPPROTO_IP, IP_TOS, &opt, sizeof(opt));
+    rc = setsockopt(config.bgp_sock, IPPROTO_IP, IP_TOS, &opt, (socklen_t) sizeof(opt));
     if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IP_TOS (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
   }
 
 #if (defined LINUX) && (defined HAVE_SO_REUSEPORT)
-  rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEADDR|SO_REUSEPORT, (char *)&yes, sizeof(yes));
+  rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEADDR|SO_REUSEPORT, (char *)&yes, (socklen_t) sizeof(yes));
   if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEADDR|SO_REUSEPORT (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
 #else
-  rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, sizeof(yes));
+  rc = setsockopt(config.bgp_sock, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, (socklen_t) sizeof(yes));
   if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for SO_REUSEADDR (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
 #endif
 
-#if (defined ENABLE_IPV6) && (defined IPV6_BINDV6ONLY)
+#if (defined IPV6_BINDV6ONLY)
   rc = setsockopt(config.bgp_sock, IPPROTO_IPV6, IPV6_BINDV6ONLY, (char *) &no, (socklen_t) sizeof(no));
   if (rc < 0) Log(LOG_ERR, "WARN ( %s/%s ): setsockopt() failed for IPV6_BINDV6ONLY (errno: %d).\n", config.name, bgp_misc_db->log_str, errno);
 #endif
 
-  if (config.nfacctd_bgp_pipe_size) {
-    int l = sizeof(config.nfacctd_bgp_pipe_size);
+  if (config.bgp_daemon_pipe_size) {
+    socklen_t l = sizeof(config.bgp_daemon_pipe_size);
     int saved = 0, obtained = 0;
 
     getsockopt(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &saved, &l);
-    Setsocksize(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &config.nfacctd_bgp_pipe_size, sizeof(config.nfacctd_bgp_pipe_size));
+    Setsocksize(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &config.bgp_daemon_pipe_size, (socklen_t) sizeof(config.bgp_daemon_pipe_size));
     getsockopt(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &obtained, &l);
 
     Setsocksize(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &saved, l);
     getsockopt(config.bgp_sock, SOL_SOCKET, SO_RCVBUF, &obtained, &l);
-    Log(LOG_INFO, "INFO ( %s/%s ): bgp_daemon_pipe_size: obtained=%d target=%d.\n", config.name, bgp_misc_db->log_str, obtained, config.nfacctd_bgp_pipe_size);
+    Log(LOG_INFO, "INFO ( %s/%s ): bgp_daemon_pipe_size: obtained=%d target=%d.\n", config.name, bgp_misc_db->log_str, obtained, config.bgp_daemon_pipe_size);
   }
 
   rc = bind(config.bgp_sock, (struct sockaddr *) &server, slen);
@@ -314,8 +351,8 @@ void skinny_bgp_daemon_online()
     char null_ip_address[] = "0.0.0.0";
     char *ip_address;
 
-    ip_address = config.nfacctd_bgp_ip ? config.nfacctd_bgp_ip : null_ip_address;
-    Log(LOG_ERR, "ERROR ( %s/%s ): bind() to ip=%s port=%d/tcp failed (errno: %d).\n", config.name, bgp_misc_db->log_str, ip_address, config.nfacctd_bgp_port, errno);
+    ip_address = config.bgp_daemon_ip ? config.bgp_daemon_ip : null_ip_address;
+    Log(LOG_ERR, "ERROR ( %s/%s ): bind() to ip=%s port=%d/tcp failed (errno: %d).\n", config.name, bgp_misc_db->log_str, ip_address, config.bgp_daemon_port, errno);
     exit_gracefully(1);
   }
 
@@ -341,12 +378,12 @@ void skinny_bgp_daemon_online()
   }
 
   /* Preparing ACL, if any */
-  if (config.nfacctd_bgp_allow_file) load_allow_file(config.nfacctd_bgp_allow_file, &allow);
+  if (config.bgp_daemon_allow_file) load_allow_file(config.bgp_daemon_allow_file, &allow);
 
   /* Preparing MD5 keys, if any */
-  if (config.nfacctd_bgp_md5_file) {
+  if (config.bgp_daemon_md5_file) {
     bgp_md5_file_init(&bgp_md5);
-    bgp_md5_file_load(config.nfacctd_bgp_md5_file, &bgp_md5);
+    bgp_md5_file_load(config.bgp_daemon_md5_file, &bgp_md5);
     if (bgp_md5.num) bgp_md5_file_process(config.bgp_sock, &bgp_md5);
   }
 
@@ -358,20 +395,77 @@ void skinny_bgp_daemon_online()
   }
 
   /* BGP peers batching checks */
-  if ((config.nfacctd_bgp_batch && !config.nfacctd_bgp_batch_interval) ||
-      (config.nfacctd_bgp_batch_interval && !config.nfacctd_bgp_batch)) {
+  if ((config.bgp_daemon_batch && !config.bgp_daemon_batch_interval) ||
+      (config.bgp_daemon_batch_interval && !config.bgp_daemon_batch)) {
     Log(LOG_WARNING, "WARN ( %s/%s ): 'bgp_daemon_batch_interval' and 'bgp_daemon_batch' both set to zero.\n", config.name, bgp_misc_db->log_str);
-    config.nfacctd_bgp_batch = 0;
-    config.nfacctd_bgp_batch_interval = 0;
+    config.bgp_daemon_batch = 0;
+    config.bgp_daemon_batch_interval = 0;
   }
-  else bgp_batch_init(&bp_batch, config.nfacctd_bgp_batch, config.nfacctd_bgp_batch_interval);
+  else bgp_batch_init(&bp_batch, config.bgp_daemon_batch, config.bgp_daemon_batch_interval);
 
   if (bgp_misc_db->msglog_backend_methods) {
 #ifdef WITH_JANSSON
-    if (!config.nfacctd_bgp_msglog_output) config.nfacctd_bgp_msglog_output = PRINT_OUTPUT_JSON;
+    if (!config.bgp_daemon_msglog_output) config.bgp_daemon_msglog_output = PRINT_OUTPUT_JSON;
 #else
     Log(LOG_WARNING, "WARN ( %s/%s ): bgp_daemon_msglog_output set to json but will produce no output (missing --enable-jansson).\n", config.name, bgp_misc_db->log_str);
 #endif
+
+#ifdef WITH_AVRO
+    if ((config.bgp_daemon_msglog_output == PRINT_OUTPUT_AVRO_BIN) ||
+	(config.bgp_daemon_msglog_output == PRINT_OUTPUT_AVRO_JSON)) {
+      assert(BGP_LOG_TYPE_MAX < MAX_AVRO_SCHEMA);
+
+      bgp_misc_db->msglog_avro_schema[0] = p_avro_schema_build_bgp(BGP_LOGDUMP_ET_LOG, "bgp_msglog");
+      bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT] = p_avro_schema_build_bgp_log_initclose(BGP_LOGDUMP_ET_LOG, "bgp_loginit");
+      bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE] = p_avro_schema_build_bgp_log_initclose(BGP_LOGDUMP_ET_LOG, "bgp_logclose");
+
+      if (config.bgp_daemon_msglog_avro_schema_file) {
+	char p_avro_schema_file[SRVBUFLEN];
+
+	if (strlen(config.bgp_daemon_msglog_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_daemon_msglog_avro_schema_file' too long. Exiting.\n", config.name, bgp_misc_db->log_str);
+	  exit_gracefully(1);
+	}
+
+	write_avro_schema_to_file_with_suffix(config.bgp_daemon_msglog_avro_schema_file, "-bgp_msglog",
+					      p_avro_schema_file, bgp_misc_db->msglog_avro_schema[0]);
+
+	write_avro_schema_to_file_with_suffix(config.bgp_daemon_msglog_avro_schema_file, "-bgp_loginit",
+					      p_avro_schema_file, bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT]);
+
+	write_avro_schema_to_file_with_suffix(config.bgp_daemon_msglog_avro_schema_file, "-bgp_logclose",
+					      p_avro_schema_file, bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE]);
+      }
+
+      if (config.bgp_daemon_msglog_kafka_avro_schema_registry) {
+#ifdef WITH_SERDES
+        if (strchr(config.bgp_daemon_msglog_kafka_topic, '$')) {
+          Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'bgp_daemon_msglog_kafka_topic' is not compatible with 'bgp_daemon_msglog_kafka_avro_schema_registry'. Exiting.\n",
+	      config.name, bgp_misc_db->log_str);
+	  exit_gracefully(1);
+        }
+
+	if (config.bgp_daemon_msglog_output == PRINT_OUTPUT_AVRO_JSON) {
+          Log(LOG_ERR, "ERROR ( %s/%s ): 'avro_json' output is not compatible with 'bgp_daemon_msglog_kafka_avro_schema_registry'. Exiting.\n",
+	      config.name, bgp_misc_db->log_str);
+	  exit_gracefully(1);
+	}
+
+	bgp_daemon_msglog_kafka_host.sd_schema[0] = compose_avro_schema_registry_name_2(config.bgp_daemon_msglog_kafka_topic, FALSE,
+										        bgp_misc_db->msglog_avro_schema[0], "bgp", "msglog",
+										        config.bgp_daemon_msglog_kafka_avro_schema_registry);
+
+	bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGINIT] = compose_avro_schema_registry_name_2(config.bgp_daemon_msglog_kafka_topic, FALSE,
+										        bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGINIT], "bgp", "loginit",
+										        config.bgp_daemon_msglog_kafka_avro_schema_registry);
+
+	bgp_daemon_msglog_kafka_host.sd_schema[BGP_LOG_TYPE_LOGCLOSE] = compose_avro_schema_registry_name_2(config.bgp_daemon_msglog_kafka_topic, FALSE,
+										        bgp_misc_db->msglog_avro_schema[BGP_LOG_TYPE_LOGCLOSE], "bgp", "logclose",
+										        config.bgp_daemon_msglog_kafka_avro_schema_registry);
+#endif
+      }
+    }
+#endif
   }
 
   if (bgp_misc_db->dump_backend_methods) {
@@ -380,6 +474,35 @@ void skinny_bgp_daemon_online()
 #else
     Log(LOG_WARNING, "WARN ( %s/%s ): bgp_table_dump_output set to json but will produce no output (missing --enable-jansson).\n", config.name, bgp_misc_db->log_str);
 #endif
+
+#ifdef WITH_AVRO
+    if ((config.bgp_table_dump_output == PRINT_OUTPUT_AVRO_BIN) ||
+	(config.bgp_table_dump_output == PRINT_OUTPUT_AVRO_JSON)) {
+      assert(BGP_LOG_TYPE_MAX < MAX_AVRO_SCHEMA);
+
+      bgp_misc_db->dump_avro_schema[0] = p_avro_schema_build_bgp(BGP_LOGDUMP_ET_DUMP, "bgp_dump");
+      bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPINIT] = p_avro_schema_build_bgp_dump_init(BGP_LOGDUMP_ET_DUMP, "bgp_dumpinit");
+      bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPCLOSE] = p_avro_schema_build_bgp_dump_close(BGP_LOGDUMP_ET_DUMP, "bgp_dumpclose");
+
+      if (config.bgp_table_dump_avro_schema_file) {
+	char p_avro_schema_file[SRVBUFLEN];
+
+	if (strlen(config.bgp_table_dump_avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	  Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_table_dump_avro_schema_file' too long. Exiting.\n", config.name, bgp_misc_db->log_str);
+	  exit_gracefully(1);
+	}
+
+	write_avro_schema_to_file_with_suffix(config.bgp_table_dump_avro_schema_file, "-bgp_dump",
+					      p_avro_schema_file, bgp_misc_db->dump_avro_schema[0]);
+
+	write_avro_schema_to_file_with_suffix(config.bgp_table_dump_avro_schema_file, "-bgp_dumpinit",
+					      p_avro_schema_file, bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPINIT]);
+
+	write_avro_schema_to_file_with_suffix(config.bgp_table_dump_avro_schema_file, "-bgp_dumpclose",
+					      p_avro_schema_file, bgp_misc_db->dump_avro_schema[BGP_LOG_TYPE_DUMPCLOSE]);
+      }
+    }
+#endif
   }
 
   if (bgp_misc_db->dump_backend_methods) {
@@ -406,19 +529,50 @@ void skinny_bgp_daemon_online()
     if (config.bgp_table_dump_kafka_topic) bgp_table_dump_init_kafka_host();
   }
 
+#ifdef WITH_AVRO
+  bgp_misc_db->avro_buf = malloc(LARGEBUFLEN);
+  if (!bgp_misc_db->avro_buf) {
+    Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, bgp_misc_db->log_str);
+    exit_gracefully(1);
+  }
+  else memset(bgp_misc_db->avro_buf, 0, LARGEBUFLEN);
+#endif
+
+  if (config.bgp_daemon_msglog_kafka_avro_schema_registry || config.bgp_table_dump_kafka_avro_schema_registry) {
+#ifndef WITH_SERDES
+    Log(LOG_ERR, "ERROR ( %s/%s ): 'bgp_*_kafka_avro_schema_registry' require --enable-serdes. Exiting.\n", config.name, bgp_misc_db->log_str);
+    exit_gracefully(1);
+#endif
+  }
+
   select_fd = bkp_select_fd = (config.bgp_sock + 1);
   recalc_fds = FALSE;
 
   bgp_link_misc_structs(bgp_misc_db);
 
+  sigemptyset(&signal_set);
+  sigaddset(&signal_set, SIGCHLD);
+  sigaddset(&signal_set, SIGHUP);
+  sigaddset(&signal_set, SIGUSR1);
+  sigaddset(&signal_set, SIGUSR2);
+  sigaddset(&signal_set, SIGTERM);
+  if (config.daemon) {
+    sigaddset(&signal_set, SIGINT);
+  }
+
   for (;;) {
     select_again:
 
+    if (!bgp_misc_db->is_thread) {
+      sigprocmask(SIG_UNBLOCK, &signal_set, NULL);
+      sigprocmask(SIG_BLOCK, &signal_set, NULL);
+    }
+
     if (recalc_fds) { 
       select_fd = config.bgp_sock;
       max_peers_idx = -1; /* .. since valid indexes include 0 */
 
-      for (peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+      for (peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
         if (select_fd < peers[peers_idx].fd) select_fd = peers[peers_idx].fd; 
 
         if (config.bgp_xconnect_map) {
@@ -454,13 +608,13 @@ void skinny_bgp_daemon_online()
 
     /* signals handling */
     if (reload_map_bgp_thread) {
-      if (config.nfacctd_bgp_allow_file) load_allow_file(config.nfacctd_bgp_allow_file, &allow);
+      if (config.bgp_daemon_allow_file) load_allow_file(config.bgp_daemon_allow_file, &allow);
 
-      if (config.nfacctd_bgp_md5_file) {
+      if (config.bgp_daemon_md5_file) {
 	bgp_md5_file_unload(&bgp_md5);
 	if (bgp_md5.num) bgp_md5_file_process(config.bgp_sock, &bgp_md5); // process unload
 
-	bgp_md5_file_load(config.nfacctd_bgp_md5_file, &bgp_md5);
+	bgp_md5_file_load(config.bgp_daemon_md5_file, &bgp_md5);
 	if (bgp_md5.num) bgp_md5_file_process(config.bgp_sock, &bgp_md5); // process load
       }
 
@@ -479,7 +633,7 @@ void skinny_bgp_daemon_online()
     }
 
     if (reload_log_bgp_thread) {
-      for (peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+      for (peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
 	if (bgp_misc_db->peers_log[peers_idx].fd) {
 	  fclose(bgp_misc_db->peers_log[peers_idx].fd);
 	  bgp_misc_db->peers_log[peers_idx].fd = open_output_file(bgp_misc_db->peers_log[peers_idx].filename, "a", FALSE);
@@ -491,6 +645,11 @@ void skinny_bgp_daemon_online()
       reload_log_bgp_thread = FALSE;
     }
 
+    if (reload_log && !bgp_misc_db->is_thread) {
+      reload_logs();
+      reload_log = FALSE;
+    }
+
     if (bgp_misc_db->msglog_backend_methods || bgp_misc_db->dump_backend_methods) {
       gettimeofday(&bgp_misc_db->log_tstamp, NULL);
       compose_timestamp(bgp_misc_db->log_tstamp_str, SRVBUFLEN, &bgp_misc_db->log_tstamp, TRUE,
@@ -520,7 +679,7 @@ void skinny_bgp_daemon_online()
       }
 
 #ifdef WITH_RABBITMQ
-      if (config.nfacctd_bgp_msglog_amqp_routing_key) { 
+      if (config.bgp_daemon_msglog_amqp_routing_key) { 
         time_t last_fail = P_broker_timers_get_last_fail(&bgp_daemon_msglog_amqp_host.btimers);
 
 	if (last_fail && ((last_fail + P_broker_timers_get_retry_interval(&bgp_daemon_msglog_amqp_host.btimers)) <= bgp_misc_db->log_tstamp.tv_sec)) {
@@ -531,7 +690,7 @@ void skinny_bgp_daemon_online()
 #endif
 
 #ifdef WITH_KAFKA
-      if (config.nfacctd_bgp_msglog_kafka_topic) {
+      if (config.bgp_daemon_msglog_kafka_topic) {
         time_t last_fail = P_broker_timers_get_last_fail(&bgp_daemon_msglog_kafka_host.btimers);
 
         if (last_fail && ((last_fail + P_broker_timers_get_retry_interval(&bgp_daemon_msglog_kafka_host.btimers)) <= bgp_misc_db->log_tstamp.tv_sec))
@@ -554,20 +713,23 @@ void skinny_bgp_daemon_online()
       fd = accept(config.bgp_sock, (struct sockaddr *) &client, &clen);
       if (fd == ERR) goto read_data;
 
-#if defined ENABLE_IPV6
       ipv4_mapped_to_ipv4(&client);
-#endif
 
       /* If an ACL is defined, here we check against and enforce it */
       if (allow.num) allowed = check_allow(&allow, (struct sockaddr *)&client);
       else allowed = TRUE;
 
       if (!allowed) {
+	char disallowed_str[INET6_ADDRSTRLEN];
+
+	sa_to_str(disallowed_str, sizeof(disallowed_str), (struct sockaddr *) &client);
+	Log(LOG_INFO, "INFO ( %s/%s ): [%s] peer '%s' not allowed. close()\n", config.name, bgp_misc_db->log_str, config.bgp_daemon_allow_file, disallowed_str);
+
         close(fd);
         goto read_data;
       }
 
-      for (peer = NULL, peers_idx = 0; peers_idx < config.nfacctd_bgp_max_peers; peers_idx++) {
+      for (peer = NULL, peers_idx = 0; peers_idx < config.bgp_daemon_max_peers; peers_idx++) {
         if (!peers[peers_idx].fd) {
 	  /*
 	     Admitted if:
@@ -606,7 +768,7 @@ void skinny_bgp_daemon_online()
       if (!peer) {
 	/* We briefly accept the new connection to be able to drop it */
         Log(LOG_ERR, "ERROR ( %s/%s ): Insufficient number of BGP peers has been configured by 'bgp_daemon_max_peers' (%d).\n",
-			config.name, bgp_misc_db->log_str, config.nfacctd_bgp_max_peers);
+			config.name, bgp_misc_db->log_str, config.bgp_daemon_max_peers);
 
 	close(fd);
 	goto read_data;
@@ -615,33 +777,23 @@ void skinny_bgp_daemon_online()
       peer->fd = fd;
       peer->idx = peers_idx; 
       FD_SET(peer->fd, &bkp_read_descs);
-      peer->addr.family = ((struct sockaddr *)&client)->sa_family;
-      if (peer->addr.family == AF_INET) {
-	peer->addr.address.ipv4.s_addr = ((struct sockaddr_in *)&client)->sin_addr.s_addr;
-	peer->tcp_port = ntohs(((struct sockaddr_in *)&client)->sin_port);
-      }
-#if defined ENABLE_IPV6
-      else if (peer->addr.family == AF_INET6) {
-	memcpy(&peer->addr.address.ipv6, &((struct sockaddr_in6 *)&client)->sin6_addr, 16);
-	peer->tcp_port = ntohs(((struct sockaddr_in6 *)&client)->sin6_port);
-      }
-#endif
+      sa_to_addr((struct sockaddr *) &client, &peer->addr, &peer->tcp_port);
 
       if (peers_cache && peers_port_cache) {
 	u_int32_t bucket;
 
-	bucket = addr_hash(&peer->addr, config.nfacctd_bgp_max_peers);
+	bucket = addr_hash(&peer->addr, config.bgp_daemon_max_peers);
 	bgp_peer_cache_insert(peers_cache, bucket, peer);
 
-	bucket = addr_port_hash(&peer->addr, peer->tcp_port, config.nfacctd_bgp_max_peers);
+	bucket = addr_port_hash(&peer->addr, peer->tcp_port, config.bgp_daemon_max_peers);
 	bgp_peer_cache_insert(peers_port_cache, bucket, peer);
       }
 
       if (bgp_misc_db->msglog_backend_methods)
-	bgp_peer_log_init(peer, config.nfacctd_bgp_msglog_output, FUNC_TYPE_BGP);
+	bgp_peer_log_init(peer, config.bgp_daemon_msglog_output, FUNC_TYPE_BGP);
 
       /* Check: more than one TCP connection from a peer (IP address) */
-      for (peers_check_idx = 0, peers_num = 0; peers_check_idx < config.nfacctd_bgp_max_peers; peers_check_idx++) { 
+      for (peers_check_idx = 0, peers_num = 0; peers_check_idx < config.bgp_daemon_max_peers; peers_check_idx++) { 
 	if (peers_idx != peers_check_idx && !memcmp(&peers[peers_check_idx].addr, &peer->addr, sizeof(peers[peers_check_idx].addr))) {
 	  int same_peer = FALSE;
 
@@ -669,9 +821,9 @@ void skinny_bgp_daemon_online()
 		bgp_peer_close(&peers[peers_check_idx], FUNC_TYPE_BGP, FALSE, FALSE, FALSE, FALSE, NULL);
 	      }
 	      else {
-		Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Refusing new connection from existing peer (residual holdtime: %u).\n",
+		Log(LOG_WARNING, "WARN ( %s/%s ): [%s] Refusing new connection from existing peer (residual holdtime: %ld).\n",
 			config.name, bgp_misc_db->log_str, bgp_peer_str,
-			(peers[peers_check_idx].ht - (now - peers[peers_check_idx].last_keepalive)));
+			(peers[peers_check_idx].ht - ((long)now - peers[peers_check_idx].last_keepalive)));
 		FD_CLR(peer->fd, &bkp_read_descs);
 		bgp_peer_close(peer, FUNC_TYPE_BGP, FALSE, FALSE, FALSE, FALSE, NULL);
 		goto read_data;
@@ -705,15 +857,15 @@ void skinny_bgp_daemon_online()
       if (!config.bgp_xconnect_map) {
         bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] BGP peers usage: %u/%u\n", config.name, bgp_misc_db->log_str,
-		bgp_peer_str, peers_num, config.nfacctd_bgp_max_peers);
+		bgp_peer_str, peers_num, config.bgp_daemon_max_peers);
       }
       else {
         bgp_peer_xconnect_print(peer, bgp_xconnect_peer_str, BGP_XCONNECT_STRLEN);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] BGP xconnects usage: %u/%u\n", config.name, bgp_misc_db->log_str,
-		bgp_xconnect_peer_str, peers_num, config.nfacctd_bgp_max_peers);
+		bgp_xconnect_peer_str, peers_num, config.bgp_daemon_max_peers);
       }
 
-      if (config.nfacctd_bgp_neighbors_file) write_neighbors_file(config.nfacctd_bgp_neighbors_file, FUNC_TYPE_BGP);
+      if (config.bgp_daemon_neighbors_file) write_neighbors_file(config.bgp_daemon_neighbors_file, FUNC_TYPE_BGP);
     }
 
     read_data:
@@ -751,8 +903,59 @@ void skinny_bgp_daemon_online()
 
     if (!peer) goto select_again;
 
-    ret = recv(recv_fd, &peer->buf.base[peer->buf.truncated_len], (peer->buf.len - peer->buf.truncated_len), 0);
-    peer->msglen = (ret + peer->buf.truncated_len);
+    if (!peer->buf.exp_len) {
+      ret = recv(recv_fd, &peer->buf.base[peer->buf.cur_len], (BGP_HEADER_SIZE - peer->buf.cur_len), 0);
+
+      if (ret > 0) {
+	peer->buf.cur_len += ret;
+
+	if (peer->buf.cur_len == BGP_HEADER_SIZE) {
+	  struct bgp_header *bhdr = (struct bgp_header *) peer->buf.base;
+
+	  if (bgp_marker_check(bhdr, BGP_MARKER_SIZE) == ERR) {
+	    bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+	    Log(LOG_INFO, "INFO ( %s/%s ): [%s] Received malformed BGP packet (marker check failed).\n",
+		config.name, bgp_misc_db->log_str, bgp_peer_str);
+
+	    peer->msglen = 0;
+	    peer->buf.cur_len = 0;
+	    peer->buf.exp_len = 0;
+	    ret = ERR;
+	  }
+	  else {
+	    peer->buf.exp_len = ntohs(bhdr->bgpo_len);
+
+	    /* commit */
+	    if (peer->buf.cur_len == peer->buf.exp_len) {
+	      peer->msglen = peer->buf.exp_len;
+	      peer->buf.cur_len = 0;
+	      peer->buf.exp_len = 0;
+	    }
+	  }
+	}
+	else {
+	  goto select_again;
+	}
+      }
+    }
+
+    if (peer->buf.exp_len) {
+      ret = recv(recv_fd, &peer->buf.base[peer->buf.cur_len], (peer->buf.exp_len - peer->buf.cur_len), 0);
+
+      if (ret > 0) {
+	peer->buf.cur_len += ret;
+
+	/* commit */
+        if (peer->buf.cur_len == peer->buf.exp_len) {
+	  peer->msglen = peer->buf.exp_len;
+	  peer->buf.cur_len = 0;
+	  peer->buf.exp_len = 0;
+	}
+	else {
+	  goto select_again;
+	}
+      }
+    }
 
     if (ret <= 0) {
       if (!config.bgp_xconnect_map) {
@@ -803,7 +1006,7 @@ void skinny_bgp_daemon_online()
 	}
       }
       else {
-	ret = send(send_fd, &peer->buf.base[peer->buf.truncated_len], peer->msglen, 0);
+	ret = send(send_fd, peer->buf.base, peer->msglen, 0);
 	if (ret <= 0) {
 	  bgp_peer_xconnect_print(peer, bgp_xconnect_peer_str, BGP_XCONNECT_STRLEN);
 
@@ -833,7 +1036,14 @@ void bgp_prepare_thread()
   memset(bgp_misc_db, 0, sizeof(struct bgp_misc_structs));
 
   bgp_misc_db->is_thread = TRUE;
-  bgp_misc_db->has_lglass = FALSE;
+  if (config.bgp_lg) bgp_misc_db->has_lglass = TRUE;
+
+  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+    bgp_misc_db->bnv = malloc(sizeof(struct bgp_node_vector));
+    memset(bgp_misc_db->bnv, 0, sizeof(struct bgp_node_vector)); 
+  }
+
+  if (config.bgp_blackhole_stdcomm_list) bgp_misc_db->has_blackhole = TRUE;
 
   bgp_misc_db->log_str = malloc(strlen("core/BGP") + 1);
   strcpy(bgp_misc_db->log_str, "core/BGP");
@@ -847,6 +1057,13 @@ void bgp_prepare_daemon()
   bgp_misc_db->is_thread = FALSE;
   if (config.bgp_lg) bgp_misc_db->has_lglass = TRUE;
 
+  if (config.rpki_roas_file || config.rpki_rtr_cache) {
+    bgp_misc_db->bnv = malloc(sizeof(struct bgp_node_vector));
+    memset(bgp_misc_db->bnv, 0, sizeof(struct bgp_node_vector));
+  }
+
+  if (config.bgp_blackhole_stdcomm_list) bgp_misc_db->has_blackhole = TRUE;
+
   bgp_misc_db->log_str = malloc(strlen("core") + 1);
   strcpy(bgp_misc_db->log_str, "core");
 }
