@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,19 +19,17 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BGP_MSG_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
 #include "bgp.h"
+#include "bgp_blackhole.h"
 
 int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
 {
   struct bgp_misc_structs *bms;
   struct bgp_msg_data bmd;
-  char tmp_packet[BGP_BUFFER_SIZE], *bgp_packet_ptr;
+  char *bgp_packet_ptr;
   char bgp_peer_str[INET6_ADDRSTRLEN];
   struct bgp_header *bhdr;
   int ret, bgp_len = 0;
@@ -55,15 +53,7 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
       return BGP_NOTIFY_HEADER_ERR;
     }
 
-    /* BGP buffer segmentation + reassembly */
-    if (peer->msglen < BGP_HEADER_SIZE || peer->msglen < (bgp_len = ntohs(bhdr->bgpo_len))) {
-      memcpy(tmp_packet, bgp_packet_ptr, peer->msglen);
-      memcpy(peer->buf.base, tmp_packet, peer->msglen);
-      peer->buf.truncated_len = peer->msglen;
-
-      break;
-    }
-    else peer->buf.truncated_len = 0;
+    bgp_len = ntohs(bhdr->bgpo_len);
 
     if (bgp_max_msglen_check(bgp_len) == ERR) {
       bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
@@ -133,10 +123,13 @@ int bgp_parse_msg(struct bgp_peer *peer, time_t now, int online)
       ret = bgp_parse_update_msg(&bmd, bgp_packet_ptr);
       if (ret < 0) {
         bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
-	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] BGP UPDATE: malformed (%d).\n", config.name, bms->log_str, bgp_peer_str);
+	Log(LOG_WARNING, "WARN ( %s/%s ): [%s] BGP UPDATE: malformed.\n", config.name, bms->log_str, bgp_peer_str);
 	return BGP_NOTIFY_UPDATE_ERR;
       }
 
+      break;
+    case BGP_ROUTE_REFRESH:
+      /* just ignore */
       break;
     default:
       bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
@@ -178,9 +171,10 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
       peer->ht = MAX(5, ntohs(bopen->bgpo_holdtime));
       peer->id.family = AF_INET; 
       peer->id.address.ipv4.s_addr = bopen->bgpo_id;
+      peer->version = BGP_VERSION4;
 
       /* Check: duplicate Router-IDs; BGP only, ie. no BMP */
-      if (bms->bgp_msg_open_router_id_check) {
+      if (!config.bgp_disable_router_id_check && bms->bgp_msg_open_router_id_check) {
 	int check_ret;
 
 	check_ret = bms->bgp_msg_open_router_id_check(bmd);
@@ -189,7 +183,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 
       /* OPEN options parsing */
       if (bopen->bgpo_optlen && bopen->bgpo_optlen >= 2) {
-	u_int8_t len, opt_type, opt_len, cap_type;
+	u_int8_t len, opt_type, opt_len;
 	char *ptr;
 
 	ptr = bgp_packet_ptr + BGP_MIN_OPEN_MSG_SIZE;
@@ -207,8 +201,8 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 	  } 
 
 	  /* 
- 	   * If we stumble upon capabilities let's curse through them to find
- 	   * some we are forced to support (ie. MP-BGP or 4-bytes AS support)
+	   * If we stumble upon capabilities let's iterate through them to find
+	   * those that we do support (ie. MP-BGP, 4-bytes AS support, etc.)
  	   */
 	  if (opt_type == BGP_OPTION_CAPABILITY) {
 	    char *optcap_ptr, *bgp_open_cap_len_reply_ptr, *bgp_open_cap_base_reply_ptr;
@@ -309,7 +303,7 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 			/* we need to send BGP_CAPABILITY_ADD_PATHS first */
 			cap_set = TRUE;
 			memcpy(bgp_open_cap_reply_ptr, optcap_ptr, 2);
-			cap_newlen_ptr = (bgp_open_cap_reply_ptr + 1);
+			cap_newlen_ptr = (u_char *)(bgp_open_cap_reply_ptr + 1);
 			(*cap_newlen_ptr) = 0;
 			bgp_open_cap_reply_ptr += 2;
 		      }
@@ -321,6 +315,16 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
 		  }
 	        }
 	      }
+	      else if (cap_type == BGP_CAPABILITY_ROUTE_REFRESH) {
+		if (config.tmp_bgp_daemon_route_refresh) {
+		  bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
+		  Log(LOG_INFO, "INFO ( %s/%s ): [%s] Capability: Route Refresh [%x]\n",
+		      config.name, bms->log_str, bgp_peer_str, cap_type);
+
+		  memcpy(bgp_open_cap_reply_ptr, optcap_ptr, cap_len+2); 
+		  bgp_open_cap_reply_ptr += cap_len+2;
+		}
+	      }
 
 	      optcap_ptr += (cap_len + 2);
 	      optcap_len -= (cap_len + 2);
@@ -372,8 +376,8 @@ int bgp_parse_open_msg(struct bgp_msg_data *bmd, char *bgp_packet_ptr, time_t no
         bgp_reply_pkt_ptr = bgp_reply_pkt;
 
         /* Replying to OPEN message */
-	if (!config.nfacctd_bgp_as) peer->myas = peer->as;
-	else peer->myas = config.nfacctd_bgp_as;
+	if (!config.bgp_daemon_as) peer->myas = peer->as;
+	else peer->myas = config.bgp_daemon_as;
 
         bgp_peer_print(peer, bgp_peer_str, INET6_ADDRSTRLEN);
         Log(LOG_INFO, "INFO ( %s/%s ): [%s] BGP_OPEN: Local AS: %u Remote AS: %u HoldTime: %u\n", config.name,
@@ -474,24 +478,24 @@ int bgp_write_open_msg(char *msg, char *cp_msg, int cp_msglen, struct bgp_peer *
   if (cp_msglen) bopen_reply->bgpo_optlen = cp_msglen;
   bopen_reply->bgpo_len = htons(BGP_MIN_OPEN_MSG_SIZE + bopen_reply->bgpo_optlen);
 
-  if (config.nfacctd_bgp_ip) str_to_addr(config.nfacctd_bgp_ip, &bgp_ip);
+  if (config.bgp_daemon_ip) str_to_addr(config.bgp_daemon_ip, &bgp_ip);
   else memset(&bgp_ip, 0, sizeof(bgp_ip));
 
-  if (config.nfacctd_bgp_id) str_to_addr(config.nfacctd_bgp_id, &bgp_id);
+  if (config.bgp_daemon_id) str_to_addr(config.bgp_daemon_id, &bgp_id);
   else memset(&bgp_id, 0, sizeof(bgp_id));
 
   /* set BGP router-ID trial #1 */
   memset(&my_id_addr, 0, sizeof(my_id_addr));
 
-  if (config.nfacctd_bgp_id && !is_any(&bgp_id) && !my_id_addr.family) {
-    my_id = config.nfacctd_bgp_id;
+  if (config.bgp_daemon_id && !is_any(&bgp_id) && !my_id_addr.family) {
+    my_id = config.bgp_daemon_id;
     str_to_addr(my_id, &my_id_addr);
     if (my_id_addr.family != AF_INET) memset(&my_id_addr, 0, sizeof(my_id_addr));
   }
 
   /* set BGP router-ID trial #2 */
-  if (config.nfacctd_bgp_ip && !is_any(&bgp_ip) && !my_id_addr.family) {
-    my_id = config.nfacctd_bgp_ip;
+  if (config.bgp_daemon_ip && !is_any(&bgp_ip) && !my_id_addr.family) {
+    my_id = config.bgp_daemon_ip;
     str_to_addr(my_id, &my_id_addr);
     if (my_id_addr.family != AF_INET) memset(&my_id_addr, 0, sizeof(my_id_addr));
   }
@@ -531,7 +535,7 @@ int bgp_write_notification_msg(char *msg, int msglen, u_int8_t n_major, u_int8_t
 
     ret += BGP_MIN_NOTIFICATION_MSG_SIZE;
 
-    /* rfc8203 (draft-ietf-idr-shutdown) */
+    /* rfc8203 */
     if (shutdown_msg) {
       shutdown_msglen = strlen(shutdown_msg);
 
@@ -570,15 +574,14 @@ int bgp_parse_notification_msg(struct bgp_msg_data *bmd, char *pkt, u_int8_t *re
   (*res_maj) = bn->bgpn_major;
   (*res_min) = bn->bgpn_minor;
 
-  /* rfc8203 (draft-ietf-idr-shutdown) */
+  /* rfc8203 */
   if (bn->bgpn_major == BGP_NOTIFY_CEASE &&
       (bn->bgpn_minor == BGP_NOTIFY_CEASE_ADMIN_SHUTDOWN || bn->bgpn_minor == BGP_NOTIFY_CEASE_ADMIN_RESET)) {
     if (rem_len) {
       pkt_ptr = (pkt + BGP_MIN_NOTIFICATION_MSG_SIZE);
       bnsm = (struct bgp_notification_shutdown_msg *) pkt_ptr;
 
-      if (bnsm->bgpnsm_len <= rem_len && bnsm->bgpnsm_len <= BGP_NOTIFY_CEASE_SM_LEN &&
-	  bnsm->bgpnsm_len < shutdown_msglen) {
+      if (bnsm->bgpnsm_len <= rem_len && bnsm->bgpnsm_len < shutdown_msglen) {
 	memcpy(shutdown_msg, bnsm->bgpnsm_data, bnsm->bgpnsm_len);
 	shutdown_msg[bnsm->bgpnsm_len] = '\0';
 	
@@ -632,7 +635,7 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   if (withdraw_len > 0) {
     withdraw.afi = AFI_IP;
     withdraw.safi = SAFI_UNICAST;
-    withdraw.nlri = pkt;
+    withdraw.nlri = (u_char *) pkt;
     withdraw.length = withdraw_len;
     pkt += withdraw_len;
   }
@@ -657,39 +660,37 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
   if (update_len > 0) {
     update.afi = AFI_IP;
     update.safi = SAFI_UNICAST;
-    update.nlri = pkt;
+    update.nlri = (u_char *) pkt;
     update.length = update_len;
   }
 
   /* NLRI parsing */
-  if (withdraw.length) bgp_nlri_parse(bmd, NULL, &withdraw);
-  if (update.length)  bgp_nlri_parse(bmd, &attr, &update);
+  if (withdraw.length) bgp_nlri_parse(bmd, NULL, &withdraw, BGP_NLRI_WITHDRAW);
+  if (update.length) bgp_nlri_parse(bmd, &attr, &update, BGP_NLRI_UPDATE);
 	
   if (mp_update.length
 	  && mp_update.afi == AFI_IP
 	  && (mp_update.safi == SAFI_UNICAST || mp_update.safi == SAFI_MPLS_LABEL ||
 	      mp_update.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, &attr, &mp_update);
+    bgp_nlri_parse(bmd, &attr, &mp_update, BGP_NLRI_UPDATE);
 
   if (mp_withdraw.length
 	  && mp_withdraw.afi == AFI_IP
 	  && (mp_withdraw.safi == SAFI_UNICAST || mp_withdraw.safi == SAFI_MPLS_LABEL ||
 	      mp_withdraw.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse (bmd, NULL, &mp_withdraw);
+    bgp_nlri_parse (bmd, NULL, &mp_withdraw, BGP_NLRI_WITHDRAW);
 
-#if defined ENABLE_IPV6
   if (mp_update.length
 	  && mp_update.afi == AFI_IP6
 	  && (mp_update.safi == SAFI_UNICAST || mp_update.safi == SAFI_MPLS_LABEL ||
 	      mp_update.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, &attr, &mp_update);
+    bgp_nlri_parse(bmd, &attr, &mp_update, BGP_NLRI_UPDATE);
 
   if (mp_withdraw.length
 	  && mp_withdraw.afi == AFI_IP6
 	  && (mp_withdraw.safi == SAFI_UNICAST || mp_withdraw.safi == SAFI_MPLS_LABEL ||
 	      mp_withdraw.safi == SAFI_MPLS_VPN))
-    bgp_nlri_parse(bmd, NULL, &mp_withdraw);
-#endif
+    bgp_nlri_parse(bmd, NULL, &mp_withdraw, BGP_NLRI_WITHDRAW);
 
   /* Receipt of End-of-RIB can be processed here; being a silent
 	 BGP receiver only, honestly it doesn't matter to us */
@@ -713,7 +714,7 @@ int bgp_parse_update_msg(struct bgp_msg_data *bmd, char *pkt)
 int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, char *ptr, int len, struct bgp_nlri *mp_update, struct bgp_nlri *mp_withdraw)
 {
   int to_the_end = len, ret;
-  u_int8_t flag, type, *tmp, mp_nlri = 0;
+  u_int8_t flag, type, *tmp;
   u_int16_t tmp16, attr_len;
   struct aspath *as4_path = NULL;
 
@@ -765,11 +766,9 @@ int bgp_attr_parse(struct bgp_peer *peer, struct bgp_attr *attr, char *ptr, int
       break;
     case BGP_ATTR_MP_REACH_NLRI:
       ret = bgp_attr_parse_mp_reach(peer, attr_len, attr, ptr, mp_update);
-      mp_nlri = TRUE;
       break;
     case BGP_ATTR_MP_UNREACH_NLRI:
       ret = bgp_attr_parse_mp_unreach(peer, attr_len, attr, ptr, mp_withdraw);
-      mp_nlri = TRUE;
       break;
     default:
       ret = 0;
@@ -837,7 +836,7 @@ int bgp_attr_parse_community(struct bgp_peer *peer, u_int16_t len, struct bgp_at
 int bgp_attr_parse_ecommunity(struct bgp_peer *peer, u_int16_t len, struct bgp_attr *attr, char *ptr, u_int8_t flag)
 {
   if (len == 0) attr->ecommunity = NULL;
-  else attr->ecommunity = (struct ecommunity *) ecommunity_parse(peer, ptr, len);
+  else attr->ecommunity = (struct ecommunity *) ecommunity_parse(peer, (u_char *) ptr, len);
 
   return SUCCESS;
 }
@@ -845,7 +844,7 @@ int bgp_attr_parse_ecommunity(struct bgp_peer *peer, u_int16_t len, struct bgp_a
 int bgp_attr_parse_lcommunity(struct bgp_peer *peer, u_int16_t len, struct bgp_attr *attr, char *ptr, u_int8_t flag)
 {
   if (len == 0) attr->lcommunity = NULL;
-  else attr->lcommunity = (struct lcommunity *) lcommunity_parse(peer, ptr, len);
+  else attr->lcommunity = (struct lcommunity *) lcommunity_parse(peer, (u_char *) ptr, len);
 
   return SUCCESS;
 }
@@ -884,7 +883,7 @@ int bgp_attr_parse_origin(struct bgp_peer *peer, u_int16_t len, struct bgp_attr
 {
   if (len != 1) return ERR;
 
-  memcpy(&attr->local_pref, ptr, 1);
+  memcpy(&attr->origin, ptr, 1);
   ptr += 1;
 
   return SUCCESS;
@@ -921,7 +920,6 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
 	attr->mp_nexthop.family = AF_INET;
 	memcpy(&attr->mp_nexthop.address.ipv4, ptr+8, 4);
 	break;
-#if defined ENABLE_IPV6
       case 16:
       case 32:
 	attr->mp_nexthop.family = AF_INET6;
@@ -932,7 +930,6 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
 	attr->mp_nexthop.family = AF_INET6;
 	memcpy(&attr->mp_nexthop.address.ipv6, ptr+8, 16);
 	break;
-#endif
       default:
 	memset(&attr->mp_nexthop, 0, sizeof(struct host_addr));
 	break;
@@ -957,7 +954,7 @@ int bgp_attr_parse_mp_reach(struct bgp_peer *peer, u_int16_t len, struct bgp_att
 
   mp_update->afi = afi;
   mp_update->safi = safi;
-  mp_update->nlri = ptr;
+  mp_update->nlri = (u_char *) ptr;
   mp_update->length = nlri_len;
 
   return SUCCESS;
@@ -982,22 +979,21 @@ int bgp_attr_parse_mp_unreach(struct bgp_peer *peer, u_int16_t len, struct bgp_a
 
   mp_withdraw->afi = afi;
   mp_withdraw->safi = safi;
-  mp_withdraw->nlri = ptr;
+  mp_withdraw->nlri = (u_char *) ptr;
   mp_withdraw->length = withdraw_len;
 
   return SUCCESS;
 }
 
-
 /* BGP UPDATE NLRI parsing */
-int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
+int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info, int type)
 {
   struct bgp_peer *peer = bmd->peer;
   u_char *pnt;
   u_char *lim;
   u_char safi, label[3];
   struct prefix p;
-  int psize, end;
+  int psize = 0, end;
   int ret;
   u_int32_t tmp32;
   u_int16_t tmp16;
@@ -1031,7 +1027,7 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
     p.prefixlen = *pnt++; end--;
     p.family = bgp_afi2family (info->afi);
 
-    if (info->safi == SAFI_UNICAST) { 
+    if (info->safi == SAFI_UNICAST) {
       if ((info->afi == AFI_IP && p.prefixlen > 32) || (info->afi == AFI_IP6 && p.prefixlen > 128)) return ERR;
 
       psize = ((p.prefixlen+7)/8);
@@ -1039,49 +1035,84 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
 
       /* Fetch prefix from NLRI packet. */
       memcpy(&p.u.prefix, pnt, psize);
-
-      // XXX: check address correctnesss now that we have it?
     }
     else if (info->safi == SAFI_MPLS_LABEL) { /* rfc3107 labeled unicast */
+      int labels_size = 0;
+      u_char *label_ptr = NULL;
+
       if ((info->afi == AFI_IP && p.prefixlen > 56) || (info->afi == AFI_IP6 && p.prefixlen > 152)) return ERR;
 
       psize = ((p.prefixlen+7)/8);
-      if (psize > end) return ERR;
+      if (psize > end || psize < 3 /* one label */) return ERR;
 
-      /* Fetch label (3) and prefix from NLRI packet */
-      memcpy(label, pnt, 3);
-      memcpy(&p.u.prefix, pnt+3, (psize-3));
-      p.prefixlen -= 24;
+      /* Fetch label(s) and prefix from NLRI packet */
+      label_ptr = pnt;
+
+      if (type == BGP_NLRI_UPDATE) {
+	while (((labels_size + 3) <= psize) && !check_bosbit(label_ptr)) {
+	  label_ptr += 3;
+	  labels_size += 3;
+	}
+      }
+
+      if ((labels_size + 3) <= psize) {
+        memcpy(label, label_ptr, 3);
+        label_ptr += 3;
+        labels_size += 3;
+      }
+      else return ERR;
+	
+      memcpy(&p.u.prefix, (pnt + labels_size), (psize - labels_size));
+      p.prefixlen -= (8 * labels_size);
     }
     else if (info->safi == SAFI_MPLS_VPN) { /* rfc4364 BGP/MPLS IP Virtual Private Networks */
+      int labels_size = 0;
+      u_char *label_ptr = NULL;
+
       if ((info->afi == AFI_IP && p.prefixlen > 120) || (info->afi == AFI_IP6 && p.prefixlen > 216)) return ERR;
 
       psize = ((p.prefixlen+7)/8);
-      if (psize > end) return ERR;
+      if (psize > end || psize < 3 /* one label */) return ERR;
 
       /* Fetch label (3), RD (8) and prefix from NLRI packet */
-      memcpy(label, pnt, 3);
+      label_ptr = pnt;
 
-      memcpy(&rd.type, pnt+3, 2);
+      if (type == BGP_NLRI_UPDATE) {
+	while (((labels_size + 3) <= psize) && !check_bosbit(label_ptr)) {
+	  label_ptr += 3;
+	  labels_size += 3;
+	}
+      }
+
+      if ((labels_size + 3) <= psize) {
+	memcpy(label, label_ptr, 3);
+	label_ptr += 3;
+	labels_size += 3;
+      }
+      else return ERR;
+
+      if (labels_size + 8 /* RD */ > psize) return ERR; 
+
+      memcpy(&rd.type, (pnt + labels_size), 2);
       rd.type = ntohs(rd.type);
       switch(rd.type) {
       case RD_TYPE_AS: 
 	rda = (struct rd_as *) &rd;
-	memcpy(&tmp16, pnt+5, 2);
-	memcpy(&tmp32, pnt+7, 4);
+	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */), 2);
+	memcpy(&tmp32, (pnt + labels_size + 2 /* RD type */ + 2 /* RD AS */), 4);
 	rda->as = ntohs(tmp16);
 	rda->val = ntohl(tmp32);
 	break;
       case RD_TYPE_IP: 
 	rdi = (struct rd_ip *) &rd;
-	memcpy(&rdi->ip.s_addr, pnt+5, 4);
-	memcpy(&tmp16, pnt+9, 2);
+	memcpy(&rdi->ip.s_addr, (pnt + labels_size + 2 /* RD type */), 4);
+	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */ + 4 /* RD IP */), 2);
 	rdi->val = ntohs(tmp16);
 	break;
       case RD_TYPE_AS4: 
 	rda4 = (struct rd_as4 *) &rd;
-	memcpy(&tmp32, pnt+5, 4);
-	memcpy(&tmp16, pnt+9, 2);
+	memcpy(&tmp32, (pnt + labels_size + 2 /* RD type */), 4);
+	memcpy(&tmp16, (pnt + labels_size + 2 /* RD type */ + 4 /* RD AS4 */), 2);
 	rda4->as = ntohl(tmp32);
 	rda4->val = ntohs(tmp16);
 	break;
@@ -1090,22 +1121,48 @@ int bgp_nlri_parse(struct bgp_msg_data *bmd, void *attr, struct bgp_nlri *info)
 	break;
       }
     
-      memcpy(&p.u.prefix, pnt+11, (psize-11));
-      p.prefixlen -= 88;
+      memcpy(&p.u.prefix, (pnt + labels_size + 8 /* RD */), (psize - (labels_size + 8 /* RD */)));
+      p.prefixlen -= (8 * (labels_size + 8 /* RD */));
     }
 
+    // XXX: check prefix correctnesss now that we have it?
+
+#if defined WITH_ZMQ
+    if (config.bgp_blackhole_stdcomm_list) {
+      bmd->is_blackhole = bgp_blackhole_evaluate_comms(attr);
+
+      /* let's process withdraws before withdrawing */  
+      if (!attr && bmd->is_blackhole) {
+	bgp_blackhole_instrument(peer, &p, attr, info->afi, safi);
+      }
+    }
+#endif
+
     /* Let's do our job now! */
-    if (attr)
+    if (attr) {
       ret = bgp_process_update(bmd, &p, attr, info->afi, safi, &rd, &path_id, label);
-    else
+    }
+    else {
       ret = bgp_process_withdraw(bmd, &p, attr, info->afi, safi, &rd, &path_id, label);
+      (void)ret; //Treat error?
+
+    }
+
+#if defined WITH_ZMQ
+    if (config.bgp_blackhole_stdcomm_list) {
+      /* let's process updates after installing */  
+      if (attr && bmd->is_blackhole) {
+	bgp_blackhole_instrument(peer, &p, attr, info->afi, safi);
+      }
+    }
+#endif
   }
 
   return SUCCESS;
 }
 
 int bgp_process_update(struct bgp_msg_data *bmd, struct prefix *p, void *attr, afi_t afi, safi_t safi,
-		       rd_t *rd, path_id_t *path_id, char *label)
+		       rd_t *rd, path_id_t *path_id, u_char *label)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_rt_structs *inter_domain_routing_db;
@@ -1240,14 +1297,14 @@ log_update:
 }
 
 int bgp_process_withdraw(struct bgp_msg_data *bmd, struct prefix *p, void *attr, afi_t afi, safi_t safi,
-			 rd_t *rd, path_id_t *path_id, char *label)
+			 rd_t *rd, path_id_t *path_id, u_char *label)
 {
   struct bgp_peer *peer = bmd->peer;
   struct bgp_rt_structs *inter_domain_routing_db;
   struct bgp_misc_structs *bms;
   struct bgp_node *route = NULL, route_local;
   struct bgp_info *ri = NULL, ri_local;
-  u_int32_t modulo;
+  u_int32_t modulo = 0;
 
   if (!peer) return ERR;
 
