@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,9 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __BGP_UTIL_C
-
 /* includes */
 #include "pmacct.h"
 #include "addr.h"
@@ -41,10 +38,41 @@ int bgp_afi2family (int afi)
 {
   if (afi == AFI_IP)
     return AF_INET;
-#ifdef ENABLE_IPV6
   else if (afi == AFI_IP6)
     return AF_INET6;
-#endif 
+  return SUCCESS;
+}
+
+int bgp_rd_ntoh(rd_t *rd)
+{
+  struct rd_ip  *rdi;
+  struct rd_as  *rda;
+  struct rd_as4 *rda4;
+
+  if (!rd) return ERR;
+
+  rd->type = ntohs(rd->type);
+
+  switch(rd->type) {
+  case RD_TYPE_AS:
+    rda = (struct rd_as *) rd;
+    rda->as = ntohs(rda->as);
+    rda->val = ntohl(rda->val);
+    break;
+  case RD_TYPE_IP:
+    rdi = (struct rd_ip *) rd;
+    rdi->val = ntohs(rdi->val);
+    break;
+  case RD_TYPE_AS4:
+    rda4 = (struct rd_as4 *) rd;
+    rda4->as = ntohl(rda4->as);
+    rda4->val = ntohs(rda4->val);
+    break;
+  default:
+    return ERR;
+    break;
+  }
+
   return SUCCESS;
 }
 
@@ -54,7 +82,7 @@ int bgp_rd2str(char *str, rd_t *rd)
   struct rd_as  *rda;
   struct rd_as4 *rda4;
   struct host_addr a;
-  u_char ip_address[INET6_ADDRSTRLEN];
+  char ip_address[INET6_ADDRSTRLEN];
 
   switch (rd->type) {
   case RD_TYPE_AS:
@@ -89,7 +117,6 @@ int bgp_str2rd(rd_t *output, char *value)
   struct host_addr a;
   char *endptr, *token;
   u_int32_t tmp32;
-  u_int16_t tmp16;
   struct rd_ip  *rdi;
   struct rd_as  *rda;
   struct rd_as4 *rda4;
@@ -166,7 +193,7 @@ int bgp_label2str(char *str, u_char *label)
   unsigned long int tmp;
   char *endp;
 
-  snprintf(str, 8, "0x%x%x%x",
+  snprintf(str, 10, "0x%02x%02x%01x",
 	(unsigned)(unsigned char)label[0],
 	(unsigned)(unsigned char)label[1],
 	(unsigned)(unsigned char)(label[2] >> 4));
@@ -225,7 +252,7 @@ struct bgp_info_extra *bgp_info_extra_get(struct bgp_info *ri)
 }
 
 struct bgp_info_extra *bgp_info_extra_process(struct bgp_peer *peer, struct bgp_info *ri, safi_t safi,
-		  			      path_id_t *path_id, rd_t *rd, char *label)
+		  			      path_id_t *path_id, rd_t *rd, u_char *label)
 {
   struct bgp_info_extra *rie = NULL;
 
@@ -331,10 +358,6 @@ unsigned int attrhash_key_make(void *p)
   key += attr->nexthop.s_addr;
   key += attr->med;
   key += attr->local_pref;
-  if (attr->pathlimit.as) {
-    key += attr->pathlimit.ttl;
-    key += attr->pathlimit.as;
-  }
 
   if (attr->aspath)
     key += aspath_key_make(attr->aspath);
@@ -364,9 +387,7 @@ int attrhash_cmp(const void *p1, const void *p2)
       && attr1->lcommunity == attr2->lcommunity
       && attr1->med == attr2->med
       && attr1->local_pref == attr2->local_pref
-      && attr1->pathlimit.ttl == attr2->pathlimit.ttl
-      && attr1->pathlimit.as == attr2->pathlimit.as
-      && !memcmp(&attr1->mp_nexthop, &attr2->mp_nexthop, sizeof(struct host_addr)))
+      && !host_addr_cmp2((struct host_addr *)&attr1->mp_nexthop, (struct host_addr *)&attr2->mp_nexthop))
     return TRUE;
 
   return FALSE;
@@ -556,7 +577,7 @@ struct bgp_peer *bgp_peer_cache_search(struct bgp_peer_cache_bucket *cache, u_in
       if (cursor->ptr->tcp_port != port) continue;
     }
 
-    if (!memcmp(&cursor->ptr->addr, ha, sizeof(struct host_addr))) {
+    if (!host_addr_cmp(&cursor->ptr->addr, ha)) {
       ret = cursor->ptr;
       break;
     }
@@ -579,14 +600,14 @@ int bgp_peer_init(struct bgp_peer *peer, int type)
   memset(peer, 0, sizeof(struct bgp_peer));
   peer->type = type;
   peer->status = Idle;
-  peer->buf.len = BGP_BUFFER_SIZE;
-  peer->buf.base = malloc(peer->buf.len);
+  peer->buf.tot_len = BGP_BUFFER_SIZE;
+  peer->buf.base = malloc(peer->buf.tot_len);
   if (!peer->buf.base) {
     Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (bgp_peer_init). Exiting ..\n", config.name, bms->log_str);
     exit_gracefully(1);
   }
   else {
-    memset(peer->buf.base, 0, peer->buf.len);
+    memset(peer->buf.base, 0, peer->buf.tot_len);
     ret = FALSE;
   }
 
@@ -741,9 +762,7 @@ void bgp_peer_xconnect_print(struct bgp_peer *peer, char *buf, int len)
 void bgp_peer_info_delete(struct bgp_peer *peer)
 {
   struct bgp_rt_structs *inter_domain_routing_db = bgp_select_routing_db(peer->type);
-  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
   struct bgp_table *table;
-  struct bgp_node *node;
   afi_t afi;
   safi_t safi;
 
@@ -752,33 +771,44 @@ void bgp_peer_info_delete(struct bgp_peer *peer)
   for (afi = AFI_IP; afi < AFI_MAX; afi++) {
     for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++) {
       table = inter_domain_routing_db->rib[afi][safi];
-      node = bgp_table_top(peer, table);
+      bgp_table_info_delete(peer, table, afi, safi);
+    }
+  }
+}
 
-      while (node) {
-        u_int32_t modulo = bms->route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
-        u_int32_t peer_buckets;
-        struct bgp_info *ri;
-        struct bgp_info *ri_next;
+void bgp_table_info_delete(struct bgp_peer *peer, struct bgp_table *table, afi_t afi, safi_t safi)
+{
+  struct bgp_misc_structs *bms = bgp_select_misc_db(peer->type);
+  struct bgp_node *node;
 
-        for (peer_buckets = 0; peer_buckets < bms->table_per_peer_buckets; peer_buckets++) {
-          for (ri = node->info[modulo+peer_buckets]; ri; ri = ri_next) {
-            if (ri->peer == peer) {
-	      if (bms->msglog_backend_methods) {
-		char event_type[] = "log";
+  node = bgp_table_top(peer, table);
 
-		bgp_peer_log_msg(node, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_DELETE);
-	      }
+  while (node) {
+    u_int32_t modulo;
+    u_int32_t peer_buckets;
+    struct bgp_info *ri;
+    struct bgp_info *ri_next;
 
-	      ri_next = ri->next; /* let's save pointer to next before free up */
-              bgp_info_delete(peer, node, ri, modulo+peer_buckets);
-            }
-	    else ri_next = ri->next;
-          }
+    if (bms->route_info_modulo) modulo = bms->route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
+    else modulo = 0;
+
+    for (peer_buckets = 0; peer_buckets < bms->table_per_peer_buckets; peer_buckets++) {
+      for (ri = node->info[modulo + peer_buckets]; ri; ri = ri_next) {
+	if (ri->peer == peer) {
+	  if (bms->msglog_backend_methods) {
+	    char event_type[] = "log";
+
+	    bgp_peer_log_msg(node, ri, afi, safi, event_type, bms->msglog_output, NULL, BGP_LOG_TYPE_DELETE);
+	  }
+
+	  ri_next = ri->next; /* let's save pointer to next before free up */
+          bgp_info_delete(peer, node, ri, (modulo + peer_buckets));
         }
-
-        node = bgp_route_next(peer, node);
+	else ri_next = ri->next;
       }
     }
+
+    node = bgp_route_next(peer, node);
   }
 }
 
@@ -792,7 +822,7 @@ int bgp_attr_munge_as4path(struct bgp_peer *peer, struct bgp_attr *attr, struct
   /* pre-requisite for AS4_PATH is AS_PATH indeed */ 
   // XXX if (as4path && !attr->aspath) return ERR;
 
-  newpath = aspath_reconcile_as4(peer, attr->aspath, as4path);
+  newpath = aspath_reconcile_as4(attr->aspath, as4path);
   aspath_unintern(peer, attr->aspath);
   attr->aspath = aspath_intern(peer, newpath);
 
@@ -860,11 +890,10 @@ void evaluate_comm_patterns(char *dst, char *src, char **patterns, int dstlen)
 {
   char *ptr, *haystack, *delim_src, *delim_ptn;
   char local_ptr[MAX_BGP_STD_COMMS], *auxptr;
-  int idx, i, j, srclen;
+  int idx, i, j;
 
   if (!src || !dst || !dstlen) return;
 
-  srclen = strlen(src);
   memset(dst, 0, dstlen);
 
   for (idx = 0, j = 0; patterns[idx]; idx++) {
@@ -925,7 +954,7 @@ void evaluate_comm_patterns(char *dst, char *src, char **patterns, int dstlen)
 
 as_t evaluate_last_asn(struct aspath *as)
 {
-  if (!as) return SUCCESS;
+  if (!as) return 0;
 
   return as->last_as;
 }
@@ -969,7 +998,7 @@ as_t evaluate_first_asn(char *src)
     is_space = FALSE;
   }
 
-  if (config.nfacctd_bgp_peer_as_skip_subas /* XXX */ && sub_as) {
+  if (config.bgp_daemon_peer_as_skip_subas /* XXX */ && sub_as) {
     while (idx < len && (src[idx] == ' ' || src[idx] == ')')) idx++;
 
     if (idx != len-1) { 
@@ -1070,7 +1099,6 @@ void write_neighbors_file(char *filename, int type)
 	  neighbor[len] = '\0';
           fwrite(neighbor, len, 1, file);
         }
-#if defined ENABLE_IPV6
 	else if (peers[idx].addr.family == AF_INET6) {
           inet_ntop(AF_INET6, &peers[idx].addr.address.ipv6, neighbor, INET6_ADDRSTRLEN);
           len = strlen(neighbor);
@@ -1078,7 +1106,6 @@ void write_neighbors_file(char *filename, int type)
           neighbor[len] = '\0';
           fwrite(neighbor, len, 1, file);
         }
-#endif
       }
     }
 
@@ -1097,9 +1124,9 @@ void bgp_config_checks(struct configuration *c)
 			  COUNT_PEER_SRC_IP|COUNT_PEER_DST_IP|COUNT_SRC_MED|COUNT_SRC_LOCAL_PREF|
 			  COUNT_MPLS_VPN_RD)) {
     /* Sanitizing the aggregation method */
-      if ( (c->what_to_count & COUNT_SRC_LOCAL_PREF && !c->nfacctd_bgp_src_local_pref_type) ||
-	   (c->what_to_count & COUNT_SRC_MED && !c->nfacctd_bgp_src_med_type) ||
-	   (c->what_to_count & COUNT_PEER_SRC_AS && !c->nfacctd_bgp_peer_as_src_type &&
+      if ( (c->what_to_count & COUNT_SRC_LOCAL_PREF && !c->bgp_daemon_src_local_pref_type) ||
+	   (c->what_to_count & COUNT_SRC_MED && !c->bgp_daemon_src_med_type) ||
+	   (c->what_to_count & COUNT_PEER_SRC_AS && !c->bgp_daemon_peer_as_src_type &&
 	     (config.acct_type != ACCT_SF && config.acct_type != ACCT_NF)) ) {
       printf("ERROR: At least one of the following primitives is in use but its source type is not specified:\n");
       printf("       peer_src_as     =>  bgp_peer_src_as_type\n");
@@ -1115,10 +1142,10 @@ void bgp_config_checks(struct configuration *c)
 			  COUNT_SRC_EXT_COMM|COUNT_SRC_AS_PATH)) ||
       (c->what_to_count_2 & (COUNT_LRG_COMM|COUNT_SRC_LRG_COMM))) {
     /* Sanitizing the aggregation method */
-    if ( (c->what_to_count & COUNT_SRC_AS_PATH && !c->nfacctd_bgp_src_as_path_type) ||
-         (c->what_to_count & COUNT_SRC_STD_COMM && !c->nfacctd_bgp_src_std_comm_type) ||
-	 (c->what_to_count & COUNT_SRC_EXT_COMM && !c->nfacctd_bgp_src_ext_comm_type) ||
-	 (c->what_to_count_2 & COUNT_SRC_LRG_COMM && !c->nfacctd_bgp_src_lrg_comm_type) ) {
+    if ( (c->what_to_count & COUNT_SRC_AS_PATH && !c->bgp_daemon_src_as_path_type) ||
+         (c->what_to_count & COUNT_SRC_STD_COMM && !c->bgp_daemon_src_std_comm_type) ||
+	 (c->what_to_count & COUNT_SRC_EXT_COMM && !c->bgp_daemon_src_ext_comm_type) ||
+	 (c->what_to_count_2 & COUNT_SRC_LRG_COMM && !c->bgp_daemon_src_lrg_comm_type) ) {
       printf("ERROR: At least one of the following primitives is in use but its source type is not specified:\n");
       printf("       src_as_path     =>  bgp_src_as_path_type\n");
       printf("       src_std_comm    =>  bgp_src_std_comm_type\n");
@@ -1202,10 +1229,12 @@ void bgp_md5_file_unload(struct bgp_md5_table *t)
 
 void bgp_md5_file_process(int sock, struct bgp_md5_table *bgp_md5)
 {
+  char peer_str[INET6_ADDRSTRLEN + PORT_STRLEN + 1];
   struct pm_tcp_md5sig md5sig;
   struct sockaddr_storage ss_md5sig, ss_server;
   struct sockaddr *sa_md5sig = (struct sockaddr *)&ss_md5sig, *sa_server = (struct sockaddr *)&ss_server;
-  int rc, keylen, idx = 0, ss_md5sig_len, ss_server_len;
+  int rc, keylen, idx = 0, ss_md5sig_len;
+  socklen_t ss_server_len;
 
   if (!bgp_md5) return;
 
@@ -1218,12 +1247,14 @@ void bgp_md5_file_process(int sock, struct bgp_md5_table *bgp_md5)
     ss_server_len = sizeof(ss_server);
     getsockname(sock, (struct sockaddr *)&ss_server, &ss_server_len);
 
-#if defined ENABLE_IPV6
-    if (sa_md5sig->sa_family == AF_INET6 && sa_server->sa_family == AF_INET)
+    if (sa_md5sig->sa_family == AF_INET6 && sa_server->sa_family == AF_INET) {
       ipv4_mapped_to_ipv4(&ss_md5sig);
-    else if (sa_md5sig->sa_family == AF_INET && sa_server->sa_family == AF_INET6)
+      ss_md5sig_len = sizeof(struct sockaddr_in);
+    }
+    else if (sa_md5sig->sa_family == AF_INET && sa_server->sa_family == AF_INET6) {
       ipv4_to_ipv4_mapped(&ss_md5sig);
-#endif
+      ss_md5sig_len = sizeof(struct sockaddr_in6);
+    }
 
     memcpy(&md5sig.tcpm_addr, &ss_md5sig, ss_md5sig_len);
     keylen = strlen(bgp_md5->table[idx].key);
@@ -1232,8 +1263,15 @@ void bgp_md5_file_process(int sock, struct bgp_md5_table *bgp_md5)
       memcpy(md5sig.tcpm_key, &bgp_md5->table[idx].key, keylen);
     }
 
-    rc = setsockopt(sock, IPPROTO_TCP, TCP_MD5SIG, &md5sig, sizeof(md5sig));
-    if (rc < 0) Log(LOG_ERR, "WARN ( %s/core/BGP ): setsockopt() failed for TCP_MD5SIG (errno: %d).\n", config.name, errno);
+    sa_to_str(peer_str, sizeof(peer_str), sa_md5sig);
+
+    rc = setsockopt(sock, IPPROTO_TCP, TCP_MD5SIG, &md5sig, (socklen_t) sizeof(md5sig));
+    if (rc < 0) {
+      Log(LOG_WARNING, "WARN ( %s/core/BGP ): setsockopt() failed for TCP_MD5SIG peer=%s (errno: %d)\n", config.name, peer_str, errno);
+    }
+    else { 
+      Log(LOG_DEBUG, "DEBUG ( %s/core/BGP ): setsockopt() set TCP_MD5SIG peer=%s\n", config.name, peer_str);
+    }
 
     idx++;
   }
@@ -1338,23 +1376,25 @@ void bgp_link_misc_structs(struct bgp_misc_structs *bms)
 #if defined WITH_KAFKA
   bms->msglog_kafka_host = &bgp_daemon_msglog_kafka_host;
 #endif
-  bms->max_peers = config.nfacctd_bgp_max_peers;
+  bms->max_peers = config.bgp_daemon_max_peers;
   bms->peers = peers;
   bms->peers_cache = peers_cache;
   bms->peers_port_cache = peers_port_cache;
   bms->xconnects = &bgp_xcs_map;
-  bms->neighbors_file = config.nfacctd_bgp_neighbors_file; 
+  bms->neighbors_file = config.bgp_daemon_neighbors_file; 
   bms->dump_file = config.bgp_table_dump_file; 
   bms->dump_amqp_routing_key = config.bgp_table_dump_amqp_routing_key; 
   bms->dump_amqp_routing_key_rr = config.bgp_table_dump_amqp_routing_key_rr;
   bms->dump_kafka_topic = config.bgp_table_dump_kafka_topic;
   bms->dump_kafka_topic_rr = config.bgp_table_dump_kafka_topic_rr;
-  bms->msglog_file = config.nfacctd_bgp_msglog_file;
-  bms->msglog_output = config.nfacctd_bgp_msglog_output;
-  bms->msglog_amqp_routing_key = config.nfacctd_bgp_msglog_amqp_routing_key;
-  bms->msglog_amqp_routing_key_rr = config.nfacctd_bgp_msglog_amqp_routing_key_rr;
-  bms->msglog_kafka_topic = config.nfacctd_bgp_msglog_kafka_topic;
-  bms->msglog_kafka_topic_rr = config.nfacctd_bgp_msglog_kafka_topic_rr;
+  bms->dump_kafka_avro_schema_registry = config.bgp_table_dump_kafka_avro_schema_registry;
+  bms->msglog_file = config.bgp_daemon_msglog_file;
+  bms->msglog_output = config.bgp_daemon_msglog_output;
+  bms->msglog_amqp_routing_key = config.bgp_daemon_msglog_amqp_routing_key;
+  bms->msglog_amqp_routing_key_rr = config.bgp_daemon_msglog_amqp_routing_key_rr;
+  bms->msglog_kafka_topic = config.bgp_daemon_msglog_kafka_topic;
+  bms->msglog_kafka_topic_rr = config.bgp_daemon_msglog_kafka_topic_rr;
+  bms->msglog_kafka_avro_schema_registry = config.bgp_daemon_msglog_kafka_avro_schema_registry;
   bms->peer_str = malloc(strlen("peer_ip_src") + 1);
   strcpy(bms->peer_str, "peer_ip_src");
   bms->peer_port_str = malloc(strlen("peer_tcp_port") + 1);
@@ -1372,8 +1412,19 @@ void bgp_link_misc_structs(struct bgp_misc_structs *bms)
 
   bms->bgp_msg_open_router_id_check = bgp_router_id_check; 
 
-  if (!bms->is_thread && !bms->dump_backend_methods && !bms->has_lglass)
+  if (!bms->is_thread && !bms->dump_backend_methods && !bms->has_lglass && !bms->has_blackhole) {
     bms->skip_rib = TRUE;
+  }
+}
+
+void bgp_blackhole_link_misc_structs(struct bgp_misc_structs *m_data)
+{
+  m_data->table_peer_buckets = 1; /* saving on DEFAULT_BGP_INFO_HASH for now */
+  m_data->table_per_peer_buckets = DEFAULT_BGP_INFO_PER_PEER_HASH;
+  m_data->table_attr_hash_buckets = HASHTABSIZE;
+  m_data->table_per_peer_hash = BGP_ASPATH_HASH_PATHID;
+  m_data->route_info_modulo = NULL;
+  m_data->bgp_lookup_node_match_cmp = NULL;
 }
 
 int bgp_peer_cmp(const void *a, const void *b)
@@ -1465,3 +1516,57 @@ int bgp_router_id_check(struct bgp_msg_data *bmd)
 
   return FALSE;
 }
+
+/*
+  utility function when an ASN is prepended by 'AS', 'AS ', etc. strings
+  that need to be stripped.
+*/
+int bgp_str2asn(char *asn_str, as_t *asn)
+{
+  char *endptr, *asn_ptr = asn_str;
+  int len, cnt;
+
+  if (!asn_str || !asn) return ERR;
+
+  len = strlen(asn_str);
+
+  for (cnt = 0; !isdigit(asn_str[cnt]) && (cnt < len); cnt++);
+
+  asn_ptr = &asn_str[cnt];
+
+  (*asn) = strtoul(asn_ptr, &endptr, 10);
+  if (endptr == asn_ptr || (*endptr) != '\0') return ERR;
+  if (errno) {
+    errno = FALSE;
+    return ERR;
+  }
+
+  return SUCCESS;
+}
+
+const char *bgp_origin_print(u_int8_t origin)
+{
+  if (origin <= BGP_ORIGIN_MAX) return bgp_origin[origin];
+  else return bgp_origin[BGP_ORIGIN_UNKNOWN];
+}
+
+u_int8_t bgp_str2origin(char *origin_str)
+{
+  if (!strcmp(origin_str, "i")) return BGP_ORIGIN_IGP;
+  else if (!strcmp(origin_str, "e")) return BGP_ORIGIN_EGP;
+  else if (!strcmp(origin_str, "u")) return BGP_ORIGIN_INCOMPLETE;
+
+  return BGP_ORIGIN_UNKNOWN;
+}
+
+u_int16_t bgp_get_packet_len(char *pkt)
+{
+  struct bgp_header *bhdr = (struct bgp_header *) pkt;
+  u_int16_t blen = 0;
+
+  if (bgp_marker_check(bhdr, BGP_MARKER_SIZE) != ERR) {
+    blen = ntohs(bhdr->bgpo_len);
+  }
+
+  return blen;
+}
