@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,8 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __KAFKA_PLUGIN_C
-
 /* includes */
 #include "pmacct.h"
 #include "pmacct-data.h"
@@ -42,12 +40,10 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   unsigned char *pipebuf;
   struct pollfd pfd;
   struct insert_data idata;
-  time_t avro_schema_deadline = 0;
-  int timeout, refresh_timeout, avro_schema_timeout = 0;
+  int timeout, refresh_timeout;
   int ret, num, recv_budget, poll_bypass;
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
   int datasize = ((struct channels_list_entry *)ptr)->datasize;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
   pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
@@ -59,11 +55,7 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   struct extra_primitives extras;
   struct primitives_ptrs prim_ptrs;
-  char *dataptr;
-
-#ifdef WITH_AVRO
-  char *avro_acct_schema_str = NULL;
-#endif
+  unsigned char *dataptr;
 
 #ifdef WITH_ZMQ
   struct p_zmq_host *zmq_host = &((struct channels_list_entry *)ptr)->zmq_host;
@@ -71,6 +63,10 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -89,32 +85,38 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   if (config.message_broker_output & PRINT_OUTPUT_JSON) {
     compose_json(config.what_to_count, config.what_to_count_2);
   }
-  else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+  else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	   (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-    avro_acct_schema = build_avro_schema(config.what_to_count, config.what_to_count_2);
-    avro_schema_add_writer_id(avro_acct_schema);
+    p_avro_acct_schema = p_avro_schema_build_acct_data(config.what_to_count, config.what_to_count_2);
+    p_avro_schema_add_writer_id(p_avro_acct_schema);
 
-    if (config.avro_schema_output_file) write_avro_schema_to_file(config.avro_schema_output_file, avro_acct_schema);
+    p_avro_acct_init_schema = p_avro_schema_build_acct_init();
+    p_avro_acct_close_schema = p_avro_schema_build_acct_close();
 
-    if (config.kafka_avro_schema_topic) {
-      if (!config.kafka_avro_schema_refresh_time)
-	config.kafka_avro_schema_refresh_time = DEFAULT_AVRO_SCHEMA_REFRESH_TIME;
+    if (config.avro_schema_file) {
+      char avro_schema_file[SRVBUFLEN];
 
-      avro_schema_deadline = time(NULL);
-      P_init_refresh_deadline(&avro_schema_deadline, config.kafka_avro_schema_refresh_time, 0, "m");
-      avro_acct_schema_str = compose_avro_purge_schema(avro_acct_schema, config.name);
-    }
-    else {
-      config.kafka_avro_schema_refresh_time = 0;
-      avro_schema_deadline = 0;
-      avro_schema_timeout = 0;
+      if (strlen(config.avro_schema_file) > (SRVBUFLEN - SUPERSHORTBUFLEN)) {
+	Log(LOG_ERR, "ERROR ( %s/%s ): 'avro_schema_file' too long. Exiting.\n", config.name, config.type);
+	exit_gracefully(1);
+      }
+
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_data", avro_schema_file, p_avro_acct_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_init", avro_schema_file, p_avro_acct_init_schema);
+      write_avro_schema_to_file_with_suffix(config.avro_schema_file, "-acct_close", avro_schema_file, p_avro_acct_close_schema);
     }
 
     if (config.kafka_avro_schema_registry) {
 #ifdef WITH_SERDES
       if (config.sql_multi_values) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): 'kafka_avro_schema_registry' is not compatible with 'kafka_multi_values'. Exiting.\n", config.name, config.type);
-        exit_gracefully(1);
+	Log(LOG_ERR, "ERROR ( %s/%s ): 'kafka_avro_schema_registry' is not compatible with 'kafka_multi_values'. Exiting.\n", config.name, config.type);
+	exit_gracefully(1);
+      }
+
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	Log(LOG_ERR, "ERROR ( %s/%s ): 'kafka_avro_schema_registry' is not compatible with 'avro_json' output. Exiting.\n", config.name, config.type);
+	exit_gracefully(1);
       }
 #else
       Log(LOG_ERR, "ERROR ( %s/%s ): 'kafka_avro_schema_registry' requires --enable-serdes. Exiting.\n", config.name, config.type);
@@ -178,6 +180,15 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   /* setting number of entries in _protocols structure */
   while (_protocols[protocols_number].number != -1) protocols_number++;
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -185,11 +196,10 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     poll_bypass = FALSE;
 
     calc_refresh_timeout(refresh_deadline, idata.now, &refresh_timeout);
-    if (config.kafka_avro_schema_topic) calc_refresh_timeout(avro_schema_deadline, idata.now, &avro_schema_timeout);
 
     pfd.fd = pipe_fd;
     pfd.events = POLLIN;
-    timeout = MIN(refresh_timeout, (avro_schema_timeout ? avro_schema_timeout : INT_MAX));
+    timeout = refresh_timeout; /* in case we have more timeouts to factor in */
     ret = poll(&pfd, (pfd.fd == ERR ? 0 : 1), timeout);
 
     if (ret <= 0) {
@@ -206,13 +216,6 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
     if (idata.now > refresh_deadline) P_cache_handle_flush_event(&pt);
 
-#ifdef WITH_AVRO
-    if (idata.now > avro_schema_deadline) {
-      kafka_avro_schema_purge(avro_acct_schema_str);
-      avro_schema_deadline += config.kafka_avro_schema_refresh_time;
-    }
-#endif
-
     recv_budget = 0;
     if (poll_bypass) {
       poll_bypass = FALSE;
@@ -250,7 +253,7 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           else {
             rg_err_count++;
             if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
                         config.name, config.type, config.buffer_size, config.pipe_size);
               Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
                         config.name, config.type);
@@ -283,7 +286,7 @@ void kafka_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
       if (config.debug_internal_msg) 
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%llu seq=%u num_entries=%u\n",
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
                 config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
                 ((struct ch_buf_hdr *)pipebuf)->num);
 
@@ -324,13 +327,13 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   struct pkt_nat_primitives *pnat = NULL;
   struct pkt_mpls_primitives *pmpls = NULL;
   struct pkt_tunnel_primitives *ptun = NULL;
-  char *pcust = NULL;
+  u_char *pcust = NULL;
   struct pkt_vlen_hdr_primitives *pvlen = NULL;
   struct pkt_bgp_primitives empty_pbgp;
   struct pkt_nat_primitives empty_pnat;
   struct pkt_mpls_primitives empty_pmpls;
   struct pkt_tunnel_primitives empty_ptun;
-  char *empty_pcust = NULL;
+  u_char *empty_pcust = NULL;
   char dyn_kafka_topic[SRVBUFLEN], *orig_kafka_topic = NULL;
   char elem_part_key[SRVBUFLEN], tmpbuf[SRVBUFLEN];
   int j, stop, is_topic_dyn = FALSE, qn = 0, ret, saved_index = index;
@@ -340,20 +343,23 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   struct pkt_data dummy_data;
   pid_t writer_pid = getpid();
 
+  //TODO solve these warnings correctly
+  (void)pvlen;
+  (void)pcust;
+  (void)ptun;
+  (void)pmpls;
+  (void)pnat;
+  (void)pbgp;
+  (void)data;
+
   char *json_buf = NULL;
   int json_buf_off = 0;
 
 #ifdef WITH_AVRO
-  avro_writer_t avro_writer;
-  char *avro_buf = NULL;
-  int avro_len = 0, avro_buffer_full = FALSE;
-#endif
-
-#ifdef WITH_SERDES
-  serdes_conf_t *sd_conf;
-  serdes_t *sd_desc;
-  serdes_schema_t *sd_schema;
-  char sd_errstr[LONGSRVBUFLEN];
+  avro_writer_t p_avro_writer = {0};
+  char *p_avro_buf = NULL;
+  int p_avro_buffer_full = FALSE;
+  size_t p_avro_len = 0;
 #endif
 
   p_kafka_init_host(&kafkap_kafka_host, config.kafka_config_file);
@@ -415,7 +421,8 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
     p_kafka_set_key(&kafkap_kafka_host, config.kafka_partition_key, config.kafka_partition_keylen);
 
   if (config.message_broker_output & PRINT_OUTPUT_JSON) p_kafka_set_content_type(&kafkap_kafka_host, PM_KAFKA_CNT_TYPE_STR);
-  else if (config.message_broker_output & PRINT_OUTPUT_AVRO) p_kafka_set_content_type(&kafkap_kafka_host, PM_KAFKA_CNT_TYPE_BIN);
+  else if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) p_kafka_set_content_type(&kafkap_kafka_host, PM_KAFKA_CNT_TYPE_BIN);
+  else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) p_kafka_set_content_type(&kafkap_kafka_host, PM_KAFKA_CNT_TYPE_STR);
   else {
     Log(LOG_ERR, "ERROR ( %s/%s ): Unsupported kafka_output value specified. Exiting.\n", config.name, config.type);
     exit_gracefully(1);
@@ -427,23 +434,28 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - START (PID: %u) ***\n", config.name, config.type, writer_pid);
   start = time(NULL);
 
-  if (config.print_markers) {
-    if (config.message_broker_output & PRINT_OUTPUT_JSON || config.message_broker_output & PRINT_OUTPUT_AVRO) {
-      void *json_obj;
-      char *json_str;
-
-      json_obj = compose_purge_init_json(config.name, writer_pid);
-
-      if (json_obj) json_str = compose_json_str(json_obj);
-      if (json_str) {
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, json_str);
-        ret = p_kafka_produce_data(&kafkap_kafka_host, json_str, strlen(json_str));
-
-        free(json_str);
-        json_str = NULL;
-      }
+#ifdef WITH_AVRO
+  if (config.kafka_avro_schema_registry) {
+#ifdef WITH_SERDES
+    if (is_topic_dyn) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): dynamic 'kafka_topic' is not compatible with 'avro_schema_registry'. Exiting.\n", config.name, config.type);
+      exit_gracefully(1);
     }
+
+    kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID] = compose_avro_schema_registry_name_2(p_kafka_get_topic(&kafkap_kafka_host),
+											  FALSE, p_avro_acct_schema, "acct", "data",
+											  config.kafka_avro_schema_registry);
+
+    kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID] = compose_avro_schema_registry_name_2(p_kafka_get_topic(&kafkap_kafka_host),
+											  FALSE, p_avro_acct_init_schema, "acct", "init",
+											  config.kafka_avro_schema_registry);
+
+    kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID] = compose_avro_schema_registry_name_2(p_kafka_get_topic(&kafkap_kafka_host),
+											   FALSE, p_avro_acct_close_schema, "acct", "close",
+											   config.kafka_avro_schema_registry);
+#endif
   }
+#endif
 
   if (config.message_broker_output & PRINT_OUTPUT_JSON) {
     if (config.sql_multi_values) {
@@ -456,49 +468,87 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
       else memset(json_buf, 0, config.sql_multi_values);
     }
   }
-  else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+  else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+           (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
     if (!config.avro_buffer_size) config.avro_buffer_size = LARGEBUFLEN;
 
-    avro_buf = malloc(config.avro_buffer_size);
+    p_avro_buf = malloc(config.avro_buffer_size);
 
-    if (!avro_buf) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (avro_buf). Exiting ..\n", config.name, config.type);
+    if (!p_avro_buf) {
+      Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (p_avro_buf). Exiting ..\n", config.name, config.type);
       exit_gracefully(1);
     }
-    else memset(avro_buf, 0, config.avro_buffer_size);
+    else {
+      memset(p_avro_buf, 0, config.avro_buffer_size);
+    }
 
-    avro_writer = avro_writer_memory(avro_buf, config.avro_buffer_size);
-
-    if (config.kafka_avro_schema_registry) {
-#ifdef WITH_SERDES
-      char *avro_acct_schema_str = write_avro_schema_to_memory(avro_acct_schema);
-      char *avro_acct_schema_name = compose_avro_schema_name(config.type, config.name);
-
-      sd_conf = serdes_conf_new(NULL, 0, "schema.registry.url", config.kafka_avro_schema_registry, NULL);
-
-      sd_desc = serdes_new(sd_conf, sd_errstr, sizeof(sd_errstr));
-      if (!sd_desc) {
-        Log(LOG_ERR, "ERROR ( %s/%s ): serdes_new() failed: %s. Exiting.\n", config.name, config.type, sd_errstr);
-        exit_gracefully(1);
-      }
-
-      sd_schema = serdes_schema_add(sd_desc, avro_acct_schema_name, -1, avro_acct_schema_str, -1, sd_errstr, sizeof(sd_errstr));
-      if (!sd_schema) {
-	Log(LOG_ERR, "ERROR ( %s/%s ): serdes_schema_add() failed: %s. Exiting.\n", config.name, config.type, sd_errstr);
-	exit_gracefully(1);
-      }
-      else {
-	Log(LOG_DEBUG, "DEBUG ( %s/%s ): serdes_schema_add(): name=%s id=%d definition=%s\n", config.name, config.type,
-		serdes_schema_name(sd_schema), serdes_schema_id(sd_schema), serdes_schema_definition(sd_schema));
-      }
-#endif
+    if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+      p_avro_writer = avro_writer_memory(p_avro_buf, config.avro_buffer_size);
     }
 #endif
   }
 
+  if (config.print_markers) {
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      void *json_obj = NULL;
+      char *json_str = NULL;
+
+      json_obj = compose_purge_init_json(config.name, writer_pid);
+
+      if (json_obj) json_str = compose_json_str(json_obj);
+      if (json_str) {
+	Log(LOG_DEBUG, "DEBUG ( %s/%s ): %s\n\n", config.name, config.type, json_str);
+	ret = p_kafka_produce_data(&kafkap_kafka_host, json_str, strlen(json_str));
+
+	free(json_str);
+	json_str = NULL;
+      }
+    }
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
+#ifdef WITH_AVRO
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_init_schema);
+      avro_value_t p_avro_value = compose_avro_acct_init(config.name, writer_pid, p_avro_iface);
+
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+	if (!config.kafka_avro_schema_registry) {
+	  p_avro_len = avro_writer_tell(p_avro_writer);
+	}
+#ifdef WITH_SERDES
+	else {
+	  void *p_avro_local_buf = NULL;
+
+	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID], &p_avro_value, &p_avro_local_buf,
+					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	    exit_gracefully(1);
+	  }
+	  else {
+	    p_avro_buf = p_avro_local_buf;
+	  }
+	}
+#endif
+        ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+        if (!config.kafka_avro_schema_registry) avro_writer_reset(p_avro_writer);
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
+
+	if (p_avro_local_buf) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_local_buf, strlen(p_avro_local_buf));
+	  free(p_avro_local_buf);
+	}
+      }
+
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
+#endif
+    }
+  }
+
   for (j = 0; j < index; j++) {
-    char *json_str;
+    char *json_str = NULL;
 
     if (queue[j]->valid != PRINT_CACHE_COMMITTED) continue;
 
@@ -542,63 +592,90 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
       json_str = compose_json_str(json_obj);
 #endif
     }
-    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      avro_value_iface_t *avro_iface = avro_generic_class_from_schema(avro_acct_schema);
-      avro_value_t avro_value = compose_avro(config.what_to_count, config.what_to_count_2, queue[j]->flow_type,
-                           &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust, pvlen, queue[j]->bytes_counter,
-                           queue[j]->packet_counter, queue[j]->flow_counter, queue[j]->tcp_flags,
-                           &queue[j]->basetime, queue[j]->stitch, avro_iface);
-      size_t avro_value_size;
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_schema);
+      avro_value_t p_avro_value = compose_avro_acct_data(config.what_to_count, config.what_to_count_2,
+			   queue[j]->flow_type, &queue[j]->primitives, pbgp, pnat, pmpls, ptun, pcust,
+			   pvlen, queue[j]->bytes_counter, queue[j]->packet_counter,
+			   queue[j]->flow_counter, queue[j]->tcp_flags, &queue[j]->basetime,
+			   queue[j]->stitch, p_avro_iface);
+      add_writer_name_and_pid_avro(p_avro_value, config.name, writer_pid);
 
-      add_writer_name_and_pid_avro(avro_value, config.name, writer_pid);
-      avro_value_sizeof(&avro_value, &avro_value_size);
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+	size_t p_avro_value_size;
 
-      if (!config.kafka_avro_schema_registry) {
-	if (avro_value_size > config.avro_buffer_size) {
-	  Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: insufficient buffer size (avro_buffer_size=%u)\n",
-	      config.name, config.type, config.avro_buffer_size);
-	  Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: increase value or look for avro_buffer_size in CONFIG-KEYS document.\n\n",
-	      config.name, config.type);
-	  exit_gracefully(1);
-	}
-	else if (avro_value_size >= (config.avro_buffer_size - avro_writer_tell(avro_writer))) {
-	  avro_buffer_full = TRUE;
-	  j--;
-	}
-	else if (avro_value_write(avro_writer, &avro_value)) {
-	  Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: unable to write value: %s\n", config.name, config.type, avro_strerror());
-	  exit_gracefully(1);
-	}
-	else mv_num++;
+        avro_value_sizeof(&p_avro_value, &p_avro_value_size);
 
-	avro_len = avro_writer_tell(avro_writer);
-      }
+	if (!config.kafka_avro_schema_registry) {
+	  if (p_avro_value_size > config.avro_buffer_size) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
+	    exit_gracefully(1);
+	  }
+	  else if (p_avro_value_size >= (config.avro_buffer_size - avro_writer_tell(p_avro_writer))) {
+	    p_avro_buffer_full = TRUE;
+	    j--;
+	  }
+	  else if (avro_value_write(p_avro_writer, &p_avro_value)) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): avro_value_write() faiiled: %s\n", config.name, config.type, avro_strerror());
+	    exit_gracefully(1);
+	  }
+	  else {
+	    mv_num++;
+	  }
+
+	  p_avro_len = avro_writer_tell(p_avro_writer);
+	}
 #ifdef WITH_SERDES
-      else {
-	char *avro_local_buf = NULL;
-
-	avro_len = 0;
-	if (avro_buf) {
-	  free(avro_buf);
-	  avro_buf = NULL;
-	}
-
-	if (serdes_schema_serialize_avro(sd_schema, &avro_value, &avro_local_buf, &avro_len, sd_errstr, sizeof(sd_errstr))) {
-	  Log(LOG_ERR, "ERROR ( %s/%s ): AVRO: serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, sd_errstr);
-	  exit_gracefully(1);
-	}
 	else {
-	  avro_buf = avro_local_buf;
-	  mv_num++;
+	  void *p_avro_local_buf = NULL;
+
+	  p_avro_len = 0;
+	  if (p_avro_buf) {
+	    free(p_avro_buf);
+	    p_avro_buf = NULL;
+	  }
+
+	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID], &p_avro_value, &p_avro_local_buf,
+					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	    exit_gracefully(1);
+	  }
+	  else {
+	    p_avro_buf = p_avro_local_buf;
+	    mv_num++;
+	  }
+	}
+#endif
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
+
+	if (p_avro_local_buf) {
+	  size_t p_avro_locbuf_len = strlen(p_avro_local_buf);
+
+	  if (p_avro_locbuf_len > config.avro_buffer_size) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): avro_buffer_size too small (%u)\n", config.name, config.type, config.avro_buffer_size);
+	    exit_gracefully(1);
+	  }
+	  else if (p_avro_locbuf_len >= (config.avro_buffer_size - strlen(p_avro_buf))) {
+	    p_avro_buffer_full = TRUE;
+	    j--;
+	  }
+	  else {
+	    strcat(p_avro_buf, p_avro_local_buf);
+	    mv_num++;
+	  }
+
+	  free(p_avro_local_buf);
 	}
       }
-#endif
 
-      avro_value_decref(&avro_value);
-      avro_value_iface_decref(avro_iface);
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
 #else
-      if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
+      if (config.debug) Log(LOG_DEBUG, "DEBUG ( %s/%s ): compose_avro_acct_data(): AVRO object not created due to missing --enable-avro\n", config.name, config.type);
 #endif
     }
 
@@ -667,9 +744,10 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
         else break;
       }
     }
-    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || avro_buffer_full) {
+      if (!config.sql_multi_values || (mv_num >= config.sql_multi_values) || p_avro_buffer_full) {
         if (is_topic_dyn) {
 	  prim_ptrs.data = &dummy_data;
 	  primptrs_set_all_from_chained_cache(&prim_ptrs, queue[j]);
@@ -683,10 +761,16 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
           p_kafka_set_topic(&kafkap_kafka_host, dyn_kafka_topic);
         }
 
-        ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, avro_len);
-        if (!config.kafka_avro_schema_registry) avro_writer_reset(avro_writer);
+	if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) { 
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+	  if (!config.kafka_avro_schema_registry) avro_writer_reset(p_avro_writer);
+	}
+	else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, strlen(p_avro_buf));
+	  memset(p_avro_buf, 0, config.avro_buffer_size);
+        }
 
-        avro_buffer_full = FALSE;
+        p_avro_buffer_full = FALSE;
         mv_num_save = mv_num;
         mv_num = 0;
 
@@ -707,13 +791,21 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 	if (!ret) qn += mv_num;
       }
     }
-    else if (config.message_broker_output & PRINT_OUTPUT_AVRO) {
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
 #ifdef WITH_AVRO
-      if (avro_len) {
-        ret = p_kafka_produce_data(&kafkap_kafka_host, avro_buf, avro_len);
-        if (!config.kafka_avro_schema_registry) avro_writer_free(avro_writer);
-
-        if (!ret) qn += mv_num;
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+	if (p_avro_len) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+	  if (!config.kafka_avro_schema_registry) avro_writer_free(p_avro_writer);
+          if (!ret) qn += mv_num;
+	}
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+        if (strlen(p_avro_buf)) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, strlen(p_avro_buf));
+	  if (!ret) qn += mv_num;
+	}
       }
 #endif
     }
@@ -722,9 +814,9 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   duration = time(NULL)-start;
 
   if (config.print_markers) {
-    if (config.message_broker_output & PRINT_OUTPUT_JSON || config.message_broker_output & PRINT_OUTPUT_AVRO) {
-      void *json_obj;
-      char *json_str;
+    if (config.message_broker_output & PRINT_OUTPUT_JSON) {
+      void *json_obj = NULL;
+      char *json_str = NULL;
 
       json_obj = compose_purge_close_json(config.name, writer_pid, qn, saved_index, duration);
 
@@ -739,11 +831,51 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
         json_str = NULL;
       }
     }
+    else if ((config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) ||
+	     (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON)) {
+#ifdef WITH_AVRO
+      avro_value_iface_t *p_avro_iface = avro_generic_class_from_schema(p_avro_acct_close_schema);
+      avro_value_t p_avro_value = compose_avro_acct_close(config.name, writer_pid, qn, saved_index, duration, p_avro_iface);
+
+      if (config.message_broker_output & PRINT_OUTPUT_AVRO_BIN) {
+	if (!config.kafka_avro_schema_registry) {
+	  p_avro_len = avro_writer_tell(p_avro_writer);
+	}
+#ifdef WITH_SERDES
+	else {
+	  void *p_avro_local_buf = NULL;
+
+	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID], &p_avro_value, &p_avro_local_buf,
+					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	    exit_gracefully(1);
+	  }
+	  else {
+	    p_avro_buf = p_avro_local_buf;
+	  }
+	}
+#endif
+	ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_buf, p_avro_len);
+	if (!config.kafka_avro_schema_registry) avro_writer_reset(p_avro_writer);
+      }
+      else if (config.message_broker_output & PRINT_OUTPUT_AVRO_JSON) {
+	char *p_avro_local_buf = write_avro_json_record_to_buf(p_avro_value);
+
+	if (p_avro_local_buf) {
+	  ret = p_kafka_produce_data(&kafkap_kafka_host, p_avro_local_buf, strlen(p_avro_local_buf));
+	  free(p_avro_local_buf);
+	}
+      }
+
+      avro_value_decref(&p_avro_value);
+      avro_value_iface_decref(p_avro_iface);
+#endif
+    }
   }
 
   p_kafka_close(&kafkap_kafka_host, FALSE);
 
-  Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %u) ***\n",
+  Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %lu) ***\n",
 		config.name, config.type, writer_pid, qn, saved_index, duration);
 
   if (config.sql_trigger_exec && !safe_action) P_trigger_exec(config.sql_trigger_exec); 
@@ -753,58 +885,6 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
   if (json_buf) free(json_buf);
 
 #ifdef WITH_AVRO
-  if (avro_buf) free(avro_buf);
+  if (p_avro_buf) free(p_avro_buf);
 #endif
 }
-
-#ifdef WITH_AVRO
-void kafka_avro_schema_purge(char *avro_schema_str)
-{
-  struct p_kafka_host kafka_avro_schema_host;
-  int part, part_cnt, tpc;
-
-  if (!avro_schema_str || !config.kafka_avro_schema_topic) return;
-
-  /* setting some defaults */
-  if (!config.sql_host) config.sql_host = default_kafka_broker_host;
-  if (!config.kafka_broker_port) config.kafka_broker_port = default_kafka_broker_port;
-
-  p_kafka_init_host(&kafka_avro_schema_host, config.kafka_config_file);
-  p_kafka_connect_to_produce(&kafka_avro_schema_host);
-  p_kafka_set_broker(&kafka_avro_schema_host, config.sql_host, config.kafka_broker_port);
-  p_kafka_set_topic(&kafka_avro_schema_host, config.kafka_avro_schema_topic);
-  p_kafka_set_partition(&kafka_avro_schema_host, config.kafka_partition);
-  p_kafka_set_key(&kafka_avro_schema_host, config.kafka_partition_key, config.kafka_partition_keylen);
-  p_kafka_set_content_type(&kafka_avro_schema_host, PM_KAFKA_CNT_TYPE_STR);
-
-  if (config.kafka_partition_dynamic) {
-    rd_kafka_resp_err_t err;
-    const struct rd_kafka_metadata *metadata;
-
-    err = rd_kafka_metadata(kafka_avro_schema_host.rk, 0, kafka_avro_schema_host.topic, &metadata, 100);
-    if (err != RD_KAFKA_RESP_ERR_NO_ERROR) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): Unable to get Kafka metadata for topic %s: %s\n",
-              config.name, config.type, kafka_avro_schema_host.topic, rd_kafka_err2str(err));
-      exit_gracefully(1);
-    }
-
-    part_cnt = -1;
-    for (tpc = 0 ; tpc < metadata->topic_cnt ; tpc++) {
-      const struct rd_kafka_metadata_topic *t = &metadata->topics[tpc];
-      if (!strcmp(t->topic, config.kafka_avro_schema_topic)) {
-          part_cnt = t->partition_cnt;
-          break;
-      }
-    }
-
-    for(part = 0; part < part_cnt; part++) {
-      p_kafka_produce_data_to_part(&kafka_avro_schema_host, avro_schema_str, strlen(avro_schema_str), part);
-    }
-  }
-  else {
-    p_kafka_produce_data(&kafka_avro_schema_host, avro_schema_str, strlen(avro_schema_str));
-  }
-
-  p_kafka_close(&kafka_avro_schema_host, FALSE);
-}
-#endif
