@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -29,8 +29,6 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#define __NFPROBE_PLUGIN_C
-
 /*
  * This is software implementation of Cisco's NetFlow(tm) traffic 
  * reporting system. It operates by listening (via libpcap) on a 
@@ -49,7 +47,6 @@
  * As this implementation watches traffic promiscuously, it is likely to 
  * place significant load on hosts or gateways on which it is installed.
  */
-
 #include "common.h"
 #include "addr.h"
 #include "sys-tree.h"
@@ -59,6 +56,8 @@
 #include "treetype.h"
 
 #include "pmacct-data.h"
+#include "net_aggr.h"
+#include "ports_aggr.h"
 #include "plugin_hooks.h"
 #include "plugin_common.h"
 
@@ -151,6 +150,12 @@ flow_compare(struct FLOW *a, struct FLOW *b)
 	if (a->port[1] != b->port[1])
 		return (ntohs(a->port[1]) > ntohs(b->port[1]) ? 1 : -1);
 
+	if (a->ifindex[0] != b->ifindex[0])
+		return (a->ifindex[0] > b->ifindex[0] ? 1 : -1);
+
+	if (a->ifindex[1] != b->ifindex[1])
+		return (a->ifindex[1] > b->ifindex[1] ? 1 : -1);
+
 	return (0);
 }
 
@@ -202,8 +207,8 @@ format_flow(struct FLOW *flow)
 	snprintf(stime, sizeof(ftime), "%s", format_time(flow->flow_start.tv_sec));
 	snprintf(ftime, sizeof(ftime), "%s", format_time(flow->flow_last.tv_sec));
 
-	snprintf(buf, sizeof(buf),  "seq:%llu [%s]:%u <> [%s]:%u proto:%u "
-	    "octets>:%llu packets>:%llu octets<:%llu packets<:%llu "
+	snprintf(buf, sizeof(buf),  "seq:%" PRIu64 " [%s]:%u <> [%s]:%u proto:%u "
+	    "octets>:%" PRIu64 " packets>:%" PRIu64 " octets<:%" PRIu64 " packets<:%" PRIu64 " "
 	    "start:%s.%03u finish:%s.%03u tcp>:%02x tcp<:%02x "
 	    "flowlabel>:%08x flowlabel<:%08x ",
 	    flow->flow_seq,
@@ -211,8 +216,8 @@ format_flow(struct FLOW *flow)
 	    (int)flow->protocol, 
 	    flow->octets[0], flow->packets[0], 
 	    flow->octets[1], flow->packets[1], 
-	    stime, (flow->flow_start.tv_usec + 500) / 1000, 
-	    ftime, (flow->flow_last.tv_usec + 500) / 1000,
+	    stime, (unsigned int)((flow->flow_start.tv_usec + 500) / 1000), 
+	    ftime, (unsigned int)((flow->flow_last.tv_usec + 500) / 1000),
 	    flow->tcp_flags[0], flow->tcp_flags[1],
 	    flow->ip6_flowlabel[0], flow->ip6_flowlabel[1]);
 
@@ -230,7 +235,7 @@ format_flow_brief(struct FLOW *flow)
 	inet_ntop(flow->af, &flow->addr[1], addr2, sizeof(addr2));
 
 	snprintf(buf, sizeof(buf), 
-	    "seq:%llu [%s]:%hu <> [%s]:%hu proto:%u",
+	    "seq:%" PRIu64 " [%s]:%hu <> [%s]:%hu proto:%u",
 	    flow->flow_seq,
 	    addr1, ntohs(flow->port[0]), addr2, ntohs(flow->port[1]),
 	    (int)flow->protocol);
@@ -323,43 +328,65 @@ l2_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int ndx)
 
   if (pmpls) flow->mpls_label[ndx] = pmpls->mpls_label_top;
 
+  /* handling input interface */
+  flow->ifindex[ndx] = 0;
+
   if (p->ifindex_in) flow->ifindex[ndx] = p->ifindex_in;
-  else if (config.nfprobe_ifindex_type && direction == DIRECTION_IN) {
-    switch (config.nfprobe_ifindex_type) {
-    case IFINDEX_STATIC:
-      flow->ifindex[ndx] = config.nfprobe_ifindex;
-      break;
-    case IFINDEX_TAG:
-      flow->ifindex[ndx] = p->tag;
-      break;
-    case IFINDEX_TAG2:
-      flow->ifindex[ndx] = p->tag2;
-      break;
-    default:
-      flow->ifindex[ndx] = 0;
-      break;
+
+  if (!flow->ifindex[ndx] || config.nfprobe_ifindex_override) {
+    if (config.nfprobe_ifindex_type && direction == DIRECTION_IN) {
+      switch (config.nfprobe_ifindex_type) {
+      case IFINDEX_STATIC:
+	if (config.nfprobe_ifindex) {
+	  flow->ifindex[ndx] = config.nfprobe_ifindex;
+	}
+
+	break;
+      case IFINDEX_TAG:
+	if (p->tag) {
+	  flow->ifindex[ndx] = p->tag;
+	}
+
+	break;
+      case IFINDEX_TAG2:
+	if (p->tag2) {
+	  flow->ifindex[ndx] = p->tag2;
+	}
+
+	break;
+      }
     }
   }
-  else flow->ifindex[ndx] = 0;
+
+  /* handling output interface */
+  flow->ifindex[ndx ^ 1] = 0;
 
   if (p->ifindex_out) flow->ifindex[ndx ^ 1] = p->ifindex_out;
-  else if (config.nfprobe_ifindex_type && direction == DIRECTION_OUT) {
-    switch (config.nfprobe_ifindex_type) {
-    case IFINDEX_STATIC:
-      flow->ifindex[ndx ^ 1] = config.nfprobe_ifindex; 
-      break;
-    case IFINDEX_TAG:
-      flow->ifindex[ndx ^ 1] = p->tag;
-      break;
-    case IFINDEX_TAG2:
-      flow->ifindex[ndx ^ 1] = p->tag2;
-      break;
-    default:
-      flow->ifindex[ndx ^ 1] = 0;
-      break;
+
+  if (!flow->ifindex[ndx ^ 1] || config.nfprobe_ifindex_override) {
+    if (config.nfprobe_ifindex_type && direction == DIRECTION_OUT) {
+      switch (config.nfprobe_ifindex_type) {
+      case IFINDEX_STATIC:
+	if (config.nfprobe_ifindex) {
+	  flow->ifindex[ndx ^ 1] = config.nfprobe_ifindex; 
+	}
+
+	break;
+      case IFINDEX_TAG:
+	if (p->tag) {
+	  flow->ifindex[ndx ^ 1] = p->tag;
+	}
+
+	break;
+      case IFINDEX_TAG2:
+	if (p->tag2) {
+	  flow->ifindex[ndx ^ 1] = p->tag2;
+	}
+
+	break;
+      }
     }
   }
-  else flow->ifindex[ndx ^ 1] = 0;
 
   return (0);
 }
@@ -406,7 +433,7 @@ ASN_to_flowrec(struct FLOW *flow, struct pkt_data *data, int ndx)
 }
 
 static int
-cust_to_flowrec(struct FLOW *flow, char *pcust, int ndx)
+cust_to_flowrec(struct FLOW *flow, u_char *pcust, int ndx)
 {
   if (pcust) {
     if (!flow->pcust[ndx]) flow->pcust[ndx] = malloc(config.cpptrs.len);
@@ -447,7 +474,7 @@ ipv4_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfra
   struct pkt_data *data = prim_ptrs->data;
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_extras *extras = prim_ptrs->pextras;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   int ndx;
@@ -488,7 +515,7 @@ ipv4_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int
 {
   struct pkt_data *data = prim_ptrs->data;
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   int ndx;
@@ -506,7 +533,6 @@ ipv4_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int
   return (0);
 }
 
-#if defined ENABLE_IPV6
 /* Convert a IPv6 packet to a partial flow record (used for comparison) */
 static int
 ipv6_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfrag, int af)
@@ -514,7 +540,7 @@ ipv6_to_flowrec(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int *isfra
   struct pkt_data *data = prim_ptrs->data;
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_extras *extras = prim_ptrs->pextras;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   int ndx;
@@ -556,7 +582,7 @@ ipv6_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int
 {
   struct pkt_data *data = prim_ptrs->data;
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
-  char *pcust = prim_ptrs->pcust;
+  u_char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   struct pkt_primitives *p = &data->primitives;
   struct in6_addr dummy_ipv6; 
@@ -575,14 +601,12 @@ ipv6_to_flowrec_update(struct FLOW *flow, struct primitives_ptrs *prim_ptrs, int
 
   return (0);
 }
-#endif 
 
 static void
 flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
 {
 	EXPIRY_REMOVE(EXPIRIES, &ft->expiries, flow->expiry);
 
-#if defined HAVE_64BIT_COUNTERS
         if (config.nfprobe_version == 9 || config.nfprobe_version == 10) {
 	  if (flow->octets[0] > (1ULL << 63) || flow->octets[1] > (1ULL << 63)) { 
                 flow->expiry->expires_at = 0;
@@ -597,14 +621,6 @@ flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
                 goto out;
           }
 	}
-#else
-	/* Flows over 2Gb traffic */
-	if (flow->octets[0] > (1U << 31) || flow->octets[1] > (1U << 31)) {
-		flow->expiry->expires_at = 0;
-		flow->expiry->reason = R_OVERBYTES;
-		goto out;
-	}
-#endif
 	
 	/* Flows over maximum life seconds */
 	if (ft->maximum_lifetime != 0 && 
@@ -654,9 +670,7 @@ flow_update_expiry(struct FLOWTRACK *ft, struct FLOW *flow)
 
 	if (ft->icmp_timeout != 0 &&
 	    ((flow->af == AF_INET && flow->protocol == IPPROTO_ICMP)
-#if defined ENABLE_IPV6	   
 	    || ((flow->af == AF_INET6 && flow->protocol == IPPROTO_ICMPV6))
-#endif
 	   )) {
 		/* UDP flows */
 		flow->expiry->expires_at = flow->flow_last.tv_sec + 
@@ -709,12 +723,10 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
     if (ipv4_to_flowrec(&tmp, prim_ptrs, &frag, af) == -1)
       goto bad;
     break;
-#if defined ENABLE_IPV6
   case AF_INET6:
     if (ipv6_to_flowrec(&tmp, prim_ptrs, &frag, af) == -1)
       goto bad;
     break;
-#endif
   default:
   bad: 
     ft->bad_packets += data->pkt_num;
@@ -768,11 +780,9 @@ process_packet(struct FLOWTRACK *ft, struct primitives_ptrs *prim_ptrs, const st
     case AF_INET:
       ipv4_to_flowrec_update(flow, prim_ptrs, &frag, af);
     break;
-#if defined ENABLE_IPV6
     case AF_INET6:
       ipv6_to_flowrec_update(flow, prim_ptrs, &frag, af);
     break;
-#endif
     }
     if (!flow->class) flow->class = tmp.class;
 #if defined (WITH_NDPI)
@@ -961,7 +971,7 @@ check_expired(struct FLOWTRACK *ft, struct NETFLOW_TARGET *target, int ex, u_int
 		    (expiry->expires_at < now.tv_sec))) {
 			/* Flow has expired */
 			if (verbose_flag)
-				Log(LOG_DEBUG, "DEBUG ( %s/%s ): Queuing flow seq:%llu (%p) for expiry\n",
+				Log(LOG_DEBUG, "DEBUG ( %s/%s ): Queuing flow seq:%" PRIu64 " (%p) for expiry\n",
 				   config.name, config.type, expiry->flow->flow_seq, expiry->flow);
 
 			/* Add to array of expired flows */
@@ -1138,9 +1148,7 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
   unsigned int h6;
   unsigned char h4;
   struct sockaddr_in *in4 = (struct sockaddr_in *)addr;
-#if defined ENABLE_IPV6
   struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)addr;
-#endif
   struct sockaddr ssource_ip;
 
   if (config.nfprobe_source_ip) {
@@ -1157,7 +1165,7 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     int opt = config.nfprobe_ipprec << 5;
     int rc;
 
-    rc = setsockopt(s, IPPROTO_IP, IP_TOS, &opt, sizeof(opt));
+    rc = setsockopt(s, IPPROTO_IP, IP_TOS, &opt, (socklen_t) sizeof(opt));
     if (rc < 0) Log(LOG_WARNING, "WARN ( %s/%s ): setsockopt() failed for IP_TOS: %s\n", config.name, config.type, strerror(errno));
   }
 
@@ -1165,7 +1173,7 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     int rc, value;
 
     value = MIN(config.pipe_size, INT_MAX); 
-    rc = Setsocksize(s, SOL_SOCKET, SO_SNDBUF, &value, sizeof(value));
+    rc = Setsocksize(s, SOL_SOCKET, SO_SNDBUF, &value, (socklen_t) sizeof(value));
     if (rc < 0) Log(LOG_WARNING, "WARN ( %s/%s ): setsockopt() failed for SOL_SNDBUF: %s\n", config.name, config.type, strerror(errno));
   }
 
@@ -1191,12 +1199,11 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     if (hoplimit == -1)
       break;
     h4 = hoplimit;
-    if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &h4, sizeof(h4)) == -1) {
+    if (setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &h4, (socklen_t) sizeof(h4)) == -1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): setsockopt() failed for IP_MULTICAST_TTL: %s\n", config.name, config.type, strerror(errno));
       exit_gracefully(1);
     }
     break;
-#if defined ENABLE_IPV6
   case AF_INET6:
     /* Default to link-local hoplimit for multicast addresses */
     if (hoplimit == -1 && IN6_IS_ADDR_MULTICAST(&in6->sin6_addr))
@@ -1204,11 +1211,10 @@ connsock(struct sockaddr_storage *addr, socklen_t len, int hoplimit)
     if (hoplimit == -1)
       break;
     h6 = hoplimit;
-    if (setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &h6, sizeof(h6)) == -1) {
+    if (setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &h6, (socklen_t) sizeof(h6)) == -1) {
       Log(LOG_ERR, "ERROR ( %s/%s ): setsockopt() failed for IPV6_MULTICAST_HOPS: %s\n", config.name, config.type, strerror(errno));
       exit_gracefully(1);
     }
-#endif
   }
 
   return(s);
@@ -1297,56 +1303,6 @@ handle_timeouts(struct FLOWTRACK *ft, char *to_spec)
   set_timeout(ft, current);
 }
 
-static void
-parse_hostport(const char *s, struct sockaddr *addr, socklen_t *len)
-{
-  char *orig, *host, *port;
-  struct addrinfo hints, *res;
-  int herr;
-
-  if ((host = orig = strdup(s)) == NULL) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), strdup() out of memory\n", config.name, config.type);
-    exit_gracefully(1);
-  }
-
-  trim_spaces(host);
-  trim_spaces(orig);
-
-  if ((port = strrchr(host, ':')) == NULL || *(++port) == '\0' || *host == '\0') {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), invalid 'nfprobe_receiver' argument\n", config.name, config.type);
-    exit_gracefully(1);
-  }
-  *(port - 1) = '\0';
-	
-  /* Accept [host]:port for numeric IPv6 addresses */
-  if (*host == '[' && *(port - 2) == ']') {
-    host++;
-    *(port - 2) = '\0';
-  }
-
-  memset(&hints, '\0', sizeof(hints));
-  hints.ai_socktype = SOCK_DGRAM;
-
-  if ((herr = getaddrinfo(host, port, &hints, &res)) == -1) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), address lookup failed\n", config.name, config.type);
-    exit_gracefully(1);
-  }
-
-  if (res == NULL || res->ai_addr == NULL) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), no addresses found for [%s]:%s\n", config.name, config.type, host, port);
-    exit_gracefully(1);
-  }
-
-  if (res->ai_addrlen > *len) {
-    Log(LOG_ERR, "ERROR ( %s/%s ): parse_hostport(), address too long.\n", config.name, config.type);
-    exit_gracefully(1);
-  }
-
-  memcpy(addr, res->ai_addr, res->ai_addrlen);
-  free(orig);
-  *len = res->ai_addrlen;
-}
-
 static void
 parse_engine(char *s, u_int8_t *engine_type, u_int32_t *engine_id)
 {
@@ -1372,11 +1328,6 @@ parse_engine(char *s, u_int8_t *engine_type, u_int32_t *engine_id)
     *engine_type = atoi(s);
     *engine_id = atoi(ptr);
     *delim = ':';
-
-    if ((*engine_type) > 255) {
-      Log(LOG_ERR, "ERROR ( %s/%s ): parse_engine(): NetFlow v5 engine_type values are limited to 0-255.\n", config.name, config.type);
-      exit_gracefully(1);
-    }
   }
   /* NetFlow v9 / IPFIX case */
   else {
@@ -1396,16 +1347,13 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   struct ports_table pt;
   struct pollfd pfd;
   unsigned char *pipebuf;
-  time_t now;
   int refresh_timeout, ret, num, recv_budget, poll_bypass;
   char default_receiver[] = "127.0.0.1:2100";
   char default_engine_v5[] = "0:0", default_engine_v9[] = "0";
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
   int datasize = ((struct channels_list_entry *)ptr)->datasize;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
-  pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
   struct networks_file_data nfd;
 
   unsigned char *rgptr, *dataptr;
@@ -1414,13 +1362,13 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   char *capfile = NULL, dest_addr[256], dest_serv[256];
   int linktype = 0, i, r, err, always_v6;
-  int max_flows, stop_collection_flag, hoplimit;
+  int max_flows, hoplimit;
   struct sockaddr_storage dest;
   struct FLOWTRACK flowtrack;
   socklen_t dest_len;
   struct NETFLOW_TARGET target;
   struct CB_CTXT cb_ctxt;
-  u_int8_t engine_type;
+  u_int8_t engine_type = 0;
   u_int32_t engine_id;
 
   struct extra_primitives extras;
@@ -1429,7 +1377,11 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 #ifdef WITH_ZMQ
   struct p_zmq_host *zmq_host = &((struct channels_list_entry *)ptr)->zmq_host;
 #else
-  void *zmq_host;
+  void *zmq_host = NULL;
+#endif
+
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
 #endif
 
   memcpy(&config, cfgptr, sizeof(struct configuration));
@@ -1463,6 +1415,7 @@ void nfprobe_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   signal(SIGCHLD, SIG_IGN);
 	    
   memset(&cb_ctxt, '\0', sizeof(cb_ctxt));
+  memset(empty_mem_area_256b, 0, sizeof(empty_mem_area_256b));
 
   init_flowtrack(&flowtrack);
 
@@ -1526,7 +1479,6 @@ sort_version:
 
   /* Main processing loop */
   gettimeofday(&flowtrack.system_boot_time, NULL);
-  stop_collection_flag = 0;
   cb_ctxt.ft = &flowtrack;
   cb_ctxt.linktype = linktype;
   cb_ctxt.want_v6 = target.dialect->v6_capable || always_v6;
@@ -1566,6 +1518,15 @@ sort_version:
     }
   }
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   for(;;) {
     status->wakeup = TRUE;
     poll_bypass = FALSE;
@@ -1597,7 +1558,10 @@ sort_version:
       reload_map = FALSE;
     }
 
-    now = time(NULL);
+    if (reload_log) {
+      reload_logs();
+      reload_log = FALSE;
+    }
 
     recv_budget = 0;
     if (poll_bypass) {
@@ -1633,7 +1597,7 @@ sort_version:
   	  else {
   	    rg_err_count++;
   	    if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
                         config.name, config.type, config.buffer_size, config.pipe_size);
               Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
                         config.name, config.type);
@@ -1666,7 +1630,7 @@ sort_version:
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
       if (config.debug_internal_msg) 
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%llu seq=%u num_entries=%u\n",
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
                 config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
                 ((struct ch_buf_hdr *)pipebuf)->num);
 
