@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,15 +19,40 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __PGSQL_PLUGIN_C
-
 /* includes */
 #include "pmacct.h"
 #include "pmacct-data.h"
 #include "plugin_hooks.h"
 #include "sql_common.h"
+#include "sql_common_m.h"
 #include "pgsql_plugin.h"
-#include "sql_common_m.c"
+
+int typed = TRUE;
+
+char pgsql_user[] = "pmacct";
+char pgsql_pwd[] = "arealsmartpwd";
+char pgsql_db[] = "pmacct";
+char pgsql_table[] = "acct";
+char pgsql_table_v2[] = "acct_v2";
+char pgsql_table_v3[] = "acct_v3";
+char pgsql_table_v4[] = "acct_v4";
+char pgsql_table_v5[] = "acct_v5";
+char pgsql_table_v6[] = "acct_v6";
+char pgsql_table_v7[] = "acct_v7";
+char pgsql_table_v8[] = "acct_v8";
+char pgsql_table_bgp[] = "acct_bgp";
+char pgsql_table_uni[] = "acct_uni";
+char pgsql_table_uni_v2[] = "acct_uni_v2";
+char pgsql_table_uni_v3[] = "acct_uni_v3";
+char pgsql_table_uni_v4[] = "acct_uni_v4";
+char pgsql_table_uni_v5[] = "acct_uni_v5";
+char pgsql_table_as[] = "acct_as";
+char pgsql_table_as_v2[] = "acct_as_v2";
+char pgsql_table_as_v3[] = "acct_as_v3";
+char pgsql_table_as_v4[] = "acct_as_v4";
+char pgsql_table_as_v5[] = "acct_as_v5";
+char typed_str[] = "typed"; 
+char unified_str[] = "unified"; 
 
 /* Functions */
 void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr) 
@@ -41,12 +66,11 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   int ret, num, recv_budget, poll_bypass;
   struct ring *rg = &((struct channels_list_entry *)ptr)->rg;
   struct ch_status *status = ((struct channels_list_entry *)ptr)->status;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
   int datasize = ((struct channels_list_entry *)ptr)->datasize;
   u_int32_t bufsz = ((struct channels_list_entry *)ptr)->bufsize;
   pid_t core_pid = ((struct channels_list_entry *)ptr)->core_pid;
   struct networks_file_data nfd;
-  char *dataptr;
+  unsigned char *dataptr;
 
   unsigned char *rgptr;
   int pollagain = TRUE;
@@ -61,6 +85,10 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
@@ -103,6 +131,15 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   sql_link_backend_descriptors(&bed, &p, &b);
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -141,7 +178,7 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     }
 
     if (idata.now > refresh_deadline) {
-      if (qq_ptr) sql_cache_flush(queries_queue, qq_ptr, &idata, FALSE);
+      if (qq_ptr) sql_cache_flush(sql_queries_queue, qq_ptr, &idata, FALSE);
       sql_cache_handle_flush_event(&idata, &refresh_deadline, &pt);
     }
     else {
@@ -194,7 +231,7 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
           else {
             rg_err_count++;
             if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+              Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
                         config.name, config.type, config.buffer_size, config.pipe_size);
               Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
                         config.name, config.type);
@@ -227,7 +264,7 @@ void pgsql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
       if (config.debug_internal_msg) 
-        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%llu seq=%u num_entries=%u\n",
+        Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
                 config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
                 ((struct ch_buf_hdr *)pipebuf)->num);
 
@@ -285,19 +322,11 @@ int PG_cache_dbop_copy(struct DBdesc *db, struct db_cache *cache_elem, struct in
     num++;
   }
 
-#if defined HAVE_64BIT_COUNTERS
-  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), "%s%llu%s%llu%s%llu\n", delim_buf, cache_elem->packet_counter,
+  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), "%s%" PRIu64 "%s%" PRIu64 "%s%" PRIu64 "\n", delim_buf, cache_elem->packet_counter,
 											delim_buf, cache_elem->bytes_counter,
 											delim_buf, cache_elem->flows_counter);
-  else snprintf(ptr_values, SPACELEFT(values_clause), "%s%llu%s%llu\n", delim_buf, cache_elem->packet_counter,
+  else snprintf(ptr_values, SPACELEFT(values_clause), "%s%" PRIu64 "%s%" PRIu64 "\n", delim_buf, cache_elem->packet_counter,
 									delim_buf, cache_elem->bytes_counter);
-#else
-  if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), "%s%lu%s%lu%s%lu\n", delim_buf, cache_elem->packet_counter,
-											delim_buf, cache_elem->bytes_counter,
-											delim_buf, cache_elem->flows_counter);
-  else snprintf(ptr_values, SPACELEFT(values_clause), "%s%lu%s%lu\n", delim_buf, cache_elem->packet_counter,
-									delim_buf, cache_elem->bytes_counter);
-#endif
 
   strncpy(sql_data, values_clause, SPACELEFT(sql_data));
 
@@ -319,8 +348,8 @@ int PG_cache_dbop_copy(struct DBdesc *db, struct db_cache *cache_elem, struct in
 
 int PG_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_data *idata)
 {
-  PGresult *ret;
-  char *ptr_values, *ptr_where, *ptr_set, *ptr_insert;
+  PGresult *ret = NULL;
+  char *ptr_values, *ptr_where, *ptr_set;
   int num=0, num_set=0, have_flows=0;
 
   if (config.what_to_count & COUNT_FLOWS) have_flows = TRUE;
@@ -329,7 +358,6 @@ int PG_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_
   ptr_where = where_clause;
   ptr_values = values_clause; 
   ptr_set = set_clause;
-  ptr_insert = insert_full_clause;
   memset(where_clause, 0, sizeof(where_clause));
   memset(values_clause, 0, sizeof(values_clause));
   memset(set_clause, 0, sizeof(set_clause));
@@ -377,13 +405,8 @@ int PG_cache_dbop(struct DBdesc *db, struct db_cache *cache_elem, struct insert_
     else {
       strncpy(insert_full_clause, insert_clause, SPACELEFT(insert_full_clause));
       strncat(insert_full_clause, insert_counters_clause, SPACELEFT(insert_full_clause));
-#if defined HAVE_64BIT_COUNTERS
-      if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), ", %llu, %llu, %llu)", cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
-      else snprintf(ptr_values, SPACELEFT(values_clause), ", %llu, %llu)", cache_elem->packet_counter, cache_elem->bytes_counter);
-#else
-      if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), ", %lu, %lu, %lu)", cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
-      else snprintf(ptr_values, SPACELEFT(values_clause), ", %lu, %lu)", cache_elem->packet_counter, cache_elem->bytes_counter);
-#endif
+      if (have_flows) snprintf(ptr_values, SPACELEFT(values_clause), ", %" PRIu64 ", %" PRIu64 ", %" PRIu64 ")", cache_elem->packet_counter, cache_elem->bytes_counter, cache_elem->flows_counter);
+      else snprintf(ptr_values, SPACELEFT(values_clause), ", %" PRIu64 ", %" PRIu64 ")", cache_elem->packet_counter, cache_elem->bytes_counter);
     }
     strncpy(sql_data, insert_full_clause, sizeof(sql_data));
     strncat(sql_data, values_clause, SPACELEFT(sql_data));
@@ -423,7 +446,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
 
   if (!index) {
     Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - START (PID: %u) ***\n", config.name, config.type, writer_pid);
-    Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: 0/0, ET: 0) ***\n", config.name, config.type, writer_pid);
+    Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: 0/0, ET: X) ***\n", config.name, config.type, writer_pid);
     return;
   }
 
@@ -449,7 +472,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   start = time(NULL);
 
   /* re-using pending queries queue stuff from parent and saving clauses */
-  memcpy(pending_queries_queue, queue, index*sizeof(struct db_cache *));
+  memcpy(sql_pending_queries_queue, queue, index*sizeof(struct db_cache *));
   pqq_ptr = index;
 
   strlcpy(orig_copy_clause, copy_clause, LONGSRVBUFLEN);
@@ -458,8 +481,8 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   strlcpy(orig_lock_clause, lock_clause, LONGSRVBUFLEN);
 
   start:
-  memcpy(queue, pending_queries_queue, pqq_ptr*sizeof(struct db_cache *));
-  memset(pending_queries_queue, 0, pqq_ptr*sizeof(struct db_cache *));
+  memcpy(queue, sql_pending_queries_queue, pqq_ptr*sizeof(struct db_cache *));
+  memset(sql_pending_queries_queue, 0, pqq_ptr*sizeof(struct db_cache *));
   index = pqq_ptr; pqq_ptr = 0;
 
   /* We check for variable substitution in SQL table */
@@ -508,7 +531,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   for (j = 0; j < index; j++) {
     go_to_pending = FALSE;
 
-    if (idata->dyn_table && (!idata->dyn_table_time_only || !config.nfacctd_time_new)) {
+    if (idata->dyn_table && (!idata->dyn_table_time_only || !config.nfacctd_time_new || (config.sql_refresh_time != idata->timeslot))) {
       time_t stamp = 0;
 
       memset(tmpbuf, 0, LONGLONGSRVBUFLEN); // XXX: pedantic?
@@ -521,7 +544,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
       pm_strftime_same(tmptable, LONGSRVBUFLEN, tmpbuf, &stamp, config.timestamps_utc);
 
       if (strncmp(idata->dyn_table_name, tmptable, SRVBUFLEN)) {
-        pending_queries_queue[pqq_ptr] = queue[idata->current_queue_elem];
+        sql_pending_queries_queue[pqq_ptr] = queue[idata->current_queue_elem];
 
         pqq_ptr++;
         go_to_pending = TRUE;
@@ -588,7 +611,7 @@ void PG_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   if (pqq_ptr) goto start;
 
   idata->elap_time = time(NULL)-start;
-  Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %u) ***\n",
+  Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %lu) ***\n",
 		config.name, config.type, writer_pid, idata->qn, saved_index, idata->elap_time);
 
   if (config.sql_trigger_exec) {
@@ -608,7 +631,7 @@ int PG_evaluate_history(int primitive)
       strncat(where[primitive].string, " AND ", sizeof(where[primitive].string));
     }
     if (!config.timestamps_since_epoch)
-      strncat(where[primitive].string, "ABSTIME(%u)::Timestamp::Timestamp without time zone = ", SPACELEFT(where[primitive].string));
+      strncat(where[primitive].string, "to_timestamp(%u)::Timestamp without time zone = ", SPACELEFT(where[primitive].string));
     else
       strncat(where[primitive].string, "%u = ", SPACELEFT(where[primitive].string));
     strncat(where[primitive].string, "stamp_inserted", SPACELEFT(where[primitive].string));
@@ -638,7 +661,7 @@ int PG_evaluate_history(int primitive)
     }
     else {
       if (!config.timestamps_since_epoch)
-	strncat(values[primitive].string, "ABSTIME(%u)::Timestamp, ABSTIME(%u)::Timestamp", SPACELEFT(values[primitive].string));
+	strncat(values[primitive].string, "to_timestamp(%u), to_timestamp(%u)", SPACELEFT(values[primitive].string));
       else
 	strncat(values[primitive].string, "%u, %u", SPACELEFT(values[primitive].string));
       values[primitive].handler = where[primitive].handler = count_timestamp_handler;
@@ -771,10 +794,10 @@ int PG_compose_static_queries()
   return primitives;
 }
 
-void PG_compose_conn_string(struct DBdesc *db, char *host, int port)
+void PG_compose_conn_string(struct DBdesc *db, char *host, int port, char *ca_file)
 {
   char *string;
-  int slen = SRVBUFLEN;
+  int slen = LONGLONGSRVBUFLEN;
   
   if (!db->conn_string) {
     db->conn_string = (char *) malloc(slen);
@@ -790,6 +813,7 @@ void PG_compose_conn_string(struct DBdesc *db, char *host, int port)
 
     if (host) snprintf(string, slen, " host=%s", host);
     if (port) snprintf(string, slen, " port=%u", port);
+    if (ca_file) snprintf(string, slen, " sslmode=verify-full sslrootcert=%s", ca_file);
   }
 }
 
@@ -860,7 +884,7 @@ void PG_create_dyn_table(struct DBdesc *db, char *buf)
   }
 }
 
-static int PG_affected_rows(PGresult *result)
+int PG_affected_rows(PGresult *result)
 {
   return atoi(PQcmdTuples(result));
 }
@@ -868,11 +892,11 @@ static int PG_affected_rows(PGresult *result)
 void PG_create_backend(struct DBdesc *db)
 {
   if (db->type == BE_TYPE_PRIMARY) {
-    PG_compose_conn_string(db, config.sql_host, config.sql_port);
+    PG_compose_conn_string(db, config.sql_host, config.sql_port, config.sql_conn_ca_file);
   }
   else if (db->type == BE_TYPE_BACKUP) {
     if (!config.sql_backup_host) return;
-    else PG_compose_conn_string(db, config.sql_backup_host, config.sql_port);
+    else PG_compose_conn_string(db, config.sql_backup_host, config.sql_port, config.sql_conn_ca_file);
   }
 }
 
@@ -976,5 +1000,9 @@ void PG_init_default_values(struct insert_data *idata)
 
 void PG_postgresql_get_version()
 {
+#if defined HAVE_PQLIBVERSION
   printf("PostgreSQL %u\n", PQlibVersion());
+#else
+  printf("PostgreSQL\n");
+#endif
 }
