@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,9 +19,6 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-/* defines */
-#define __PMACCTD_C
-
 /* includes */
 #include "pmacct.h"
 #include "pmacct-data.h"
@@ -36,8 +33,10 @@
 #include "bgp/bgp.h"
 #include "classifier.h"
 #include "isis/isis.h"
+#include "bmp/bmp.h"
 #if defined (WITH_NDPI)
 #include "ndpi/ndpi.h"
+#include "ndpi/ndpi_util.h"
 #endif
 #include "jhash.h"
 
@@ -84,6 +83,7 @@ void usage_daemon(char *prog_name)
   printf("  -r  \tRefresh time (in seconds)\n");
   printf("  -O  \t[ formatted | csv | json | avro ] \n\tOutput format\n");
   printf("  -o  \tPath to output file\n");
+  printf("  -M  \tPrint event init/close marker messages\n");
   printf("  -A  \tAppend output (applies to -o)\n");
   printf("  -E  \tCSV format separator (applies to -O csv, DEFAULT: ',')\n");
   printf("\n");
@@ -94,23 +94,18 @@ void usage_daemon(char *prog_name)
   printf("For suggestions, critics, bugs, contact me: %s.\n", MANTAINER);
 }
 
-void pm_pcap_device_initialize(struct pcap_devices *map)
+void pm_pcap_device_copy_all(struct pm_pcap_devices *dst, struct pm_pcap_devices *src)
 {
-  memset(map, 0, sizeof(struct pcap_devices));
+  memcpy(dst, src, sizeof(struct pm_pcap_devices));
 }
 
-void pm_pcap_device_copy_all(struct pcap_devices *dst, struct pcap_devices *src)
+void pm_pcap_device_copy_entry(struct pm_pcap_devices *dst, struct pm_pcap_devices *src, int src_idx)
 {
-  memcpy(dst, src, sizeof(struct pcap_devices));
-}
-
-void pm_pcap_device_copy_entry(struct pcap_devices *dst, struct pcap_devices *src, int src_idx)
-{
-  memcpy(&dst->list[dst->num], &src->list[src_idx], sizeof(struct pcap_device));
+  memcpy(&dst->list[dst->num], &src->list[src_idx], sizeof(struct pm_pcap_device));
   dst->num++;
 }
 
-int pm_pcap_device_getindex_byifname(struct pcap_devices *map, char *ifname)
+int pm_pcap_device_getindex_byifname(struct pm_pcap_devices *map, char *ifname)
 {
   int loc_idx;
    for (loc_idx = 0; loc_idx < map->num; loc_idx++) {
@@ -129,86 +124,55 @@ pcap_t *pm_pcap_open(const char *dev_ptr, int snaplen, int promisc,
   int ret;
 
   p = pcap_create(dev_ptr, errbuf);
-  if (p == NULL)
-    return NULL;
+  if (p == NULL) return NULL;
 
   ret = pcap_set_snaplen(p, snaplen);
-  if (ret < 0)
-    goto err;
+  if (ret < 0) goto err;
 
   ret = pcap_set_promisc(p, promisc);
-  if (ret < 0)
-    goto err;
+  if (ret < 0) goto err;
 
   ret = pcap_set_timeout(p, to_ms);
-  if (ret < 0)
-    goto err;
+  if (ret < 0) goto err;
 
 #ifdef PCAP_SET_PROTOCOL
   ret = pcap_set_protocol(p, protocol);
-  if (ret < 0)
-    goto err;
+  if (ret < 0) goto err;
 #else
-  if (protocol)
+  if (protocol) {
     Log(LOG_WARNING, "WARN ( %s/core ): pcap_protocol specified but linked against a version of libpcap that does not support pcap_set_protocol().\n", config.name);
+  }
 #endif
 
-  /* XXX: rely on external filtering for now */
-/* 
-  ret = pcap_setdirection(p, direction);
-  if (ret < 0 && direction != PCAP_D_INOUT)
-    Log(LOG_WARNING, "INFO ( %s/core ): direction specified but linked against a version of libpcap that does not support pcap_setdirection().\n", config.name);
-*/
-
   ret = pcap_activate(p);
-  if (ret < 0)
-    goto err;
+  if (ret < 0) goto err;
+
+#ifdef PCAP_SET_DIRECTION
+  ret = pcap_setdirection(p, direction);
+  if (ret < 0) goto err;
+#endif
 
   return p;
 
 err:
-  if (ret == PCAP_ERROR)
+  if (ret == PCAP_ERROR) {
     snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s", dev_ptr, pcap_geterr(p));
+  }
   else if (ret == PCAP_ERROR_NO_SUCH_DEVICE ||
 	   ret == PCAP_ERROR_PERM_DENIED ||
-	   ret == PCAP_ERROR_PROMISC_PERM_DENIED)
-    snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s (%s)", dev_ptr,
-	     pcap_statustostr(ret), pcap_geterr(p));
-  else
-    snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s", dev_ptr,
-	     pcap_statustostr(ret));
+	   ret == PCAP_ERROR_PROMISC_PERM_DENIED) {
+    snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s (%s)", dev_ptr, pcap_statustostr(ret), pcap_geterr(p));
+  }
+  else {
+    snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s", dev_ptr, pcap_statustostr(ret));
+  }
 
   pcap_close(p);
 
   return NULL;
 }
 
-void pm_pcap_add_filter(struct pcap_device *dev_ptr)
-{
-  /* pcap library stuff */
-  bpf_u_int32 localnet, netmask;
-  struct bpf_program filter;
-  char errbuf[PCAP_ERRBUF_SIZE];
-
-  if (!dev_ptr->str || pcap_lookupnet(dev_ptr->str, &localnet, &netmask, errbuf) < 0) {
-    localnet = 0;
-    netmask = PCAP_NETMASK_UNKNOWN;
-    if (dev_ptr->str) Log(LOG_WARNING, "WARN ( %s/core ): %s\n", config.name, errbuf);
-  }
-
-  memset(&filter, 0, sizeof(filter));
-  if (pcap_compile(dev_ptr->dev_desc, &filter, config.clbuf, 0, netmask) < 0) {
-    Log(LOG_WARNING, "WARN ( %s/core ): %s (going on without a filter)\n", config.name, pcap_geterr(dev_ptr->dev_desc));
-  }
-  else {
-    if (pcap_setfilter(dev_ptr->dev_desc, &filter) < 0) {
-      Log(LOG_WARNING, "WARN ( %s/core ): %s (going on without a filter)\n", config.name, pcap_geterr(dev_ptr->dev_desc));
-    }
-    else pcap_freecode(&filter);
-  }
-}
-
-int pm_pcap_add_interface(struct pcap_device *dev_ptr, char *ifname, struct pcap_interface *pcap_if_entry, int psize)
+int pm_pcap_add_interface(struct pm_pcap_device *dev_ptr, char *ifname, struct pm_pcap_interface *pm_pcap_if_entry, int psize)
 {
   /* pcap library stuff */
   char errbuf[PCAP_ERRBUF_SIZE];
@@ -217,7 +181,7 @@ int pm_pcap_add_interface(struct pcap_device *dev_ptr, char *ifname, struct pcap
   int ret = SUCCESS, attempts = FALSE, index;
   int direction;
 
-  if (pcap_if_entry && pcap_if_entry->direction) direction = pcap_if_entry->direction; 
+  if (pm_pcap_if_entry && pm_pcap_if_entry->direction) direction = pm_pcap_if_entry->direction; 
   else direction = config.pcap_direction;
 
   throttle_startup:
@@ -235,8 +199,8 @@ int pm_pcap_add_interface(struct pcap_device *dev_ptr, char *ifname, struct pcap
     }
 
     dev_ptr->active = TRUE;
-    dev_ptr->pcap_if = pcap_if_entry;
-    strncpy(dev_ptr->str, ifname, strlen(ifname));
+    dev_ptr->pcap_if = pm_pcap_if_entry;
+    strncpy(dev_ptr->str, ifname, (sizeof(dev_ptr->str) - 1));
 
     if (config.pcap_ifindex == PCAP_IFINDEX_SYS)
       dev_ptr->id = if_nametoindex(dev_ptr->str);
@@ -244,7 +208,7 @@ int pm_pcap_add_interface(struct pcap_device *dev_ptr, char *ifname, struct pcap
       dev_ptr->id = jhash(dev_ptr->str, strlen(dev_ptr->str), 0);
     else if (config.pcap_ifindex == PCAP_IFINDEX_MAP) {
       if (config.pcap_interfaces_map) {
-	dev_ptr->id = pcap_interfaces_map_lookup_ifname(&pcap_if_map, dev_ptr->str);
+	dev_ptr->id = pm_pcap_interfaces_map_lookup_ifname(&pm_pcap_if_map, dev_ptr->str);
       }
       else {
 	Log(LOG_ERR, "ERROR ( %s/core ): pcap_ifindex set to 'map' but no pcap_interface_map is defined. Exiting.\n", config.name);
@@ -256,9 +220,10 @@ int pm_pcap_add_interface(struct pcap_device *dev_ptr, char *ifname, struct pcap
     dev_ptr->fd = pcap_fileno(dev_ptr->dev_desc);
 
     if (config.nfacctd_pipe_size) {
-      int slen = sizeof(config.nfacctd_pipe_size), x;
-
 #if defined (PCAP_TYPE_linux) || (PCAP_TYPE_snoop)
+      socklen_t slen = sizeof(config.nfacctd_pipe_size);
+      int x;
+
       Setsocksize(pcap_fileno(dev_ptr->dev_desc), SOL_SOCKET, SO_RCVBUF, &config.nfacctd_pipe_size, slen);
       getsockopt(pcap_fileno(dev_ptr->dev_desc), SOL_SOCKET, SO_RCVBUF, &x, &slen);
       Log(LOG_DEBUG, "DEBUG ( %s/core ): pmacctd_pipe_size: obtained=%d target=%d.\n", config.name, x, config.nfacctd_pipe_size);
@@ -304,12 +269,12 @@ int pm_pcap_add_interface(struct pcap_device *dev_ptr, char *ifname, struct pcap
 int main(int argc,char **argv, char **envp)
 {
   /* pcap library stuff */
-  char errbuf[PCAP_ERRBUF_SIZE];
   struct pcap_pkthdr pkt_hdr;
   const u_char *pkt_body;
+  pcap_if_t *pm_pcap_ifs = NULL;
 
-  int index, index_rr, logf, ret;
-  int pcap_savefile_round = 0;
+  int index, index_rr = 0, logf, ret;
+  int pm_pcap_savefile_round = 0;
 
   struct plugins_list_entry *list;
   struct plugin_requests req;
@@ -322,22 +287,22 @@ int main(int argc,char **argv, char **envp)
   struct id_table biss_table;
   struct id_table bta_table;
 
-  struct pcap_device *dev_ptr;
-  struct pcap_callback_data cb_data;
+  struct pm_pcap_device *dev_ptr;
+  struct pm_pcap_callback_data cb_data;
 
   /* select() stuff */
   fd_set read_descs, bkp_read_descs;
-  int select_fd, bkp_select_fd, select_num;
+  int select_fd, bkp_select_fd;
 
   /* getopt() stuff */
   extern char *optarg;
   extern int optind, opterr, optopt;
   int errflag, cp;
 
-#if defined ENABLE_IPV6
   struct sockaddr_storage client;
-#else
-  struct sockaddr client;
+
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
 #endif
 
 #if defined HAVE_MALLOPT
@@ -350,6 +315,7 @@ int main(int argc,char **argv, char **envp)
   /* a bunch of default definitions */
   reload_map = FALSE;
   reload_map_pmacctd = FALSE;
+  print_stats = FALSE;
   reload_geoipv2_file = FALSE;
   bpas_map_allocated = FALSE;
   blp_map_allocated = FALSE;
@@ -379,8 +345,10 @@ int main(int argc,char **argv, char **envp)
   memset(&cb_data, 0, sizeof(cb_data));
   memset(&tunnel_registry, 0, sizeof(tunnel_registry));
   memset(&reload_map_tstamp, 0, sizeof(reload_map_tstamp));
-  pm_pcap_device_initialize(&device);
-  pm_pcap_device_initialize(&bkp_device);
+  memset(&device, 0, sizeof(device));
+  memset(empty_mem_area_256b, 0, sizeof(empty_mem_area_256b));
+  pm_pcap_device_initialize(&devices);
+  pm_pcap_device_initialize(&bkp_devices);
   log_notifications_init(&log_notifications);
   config.acct_type = ACCT_PM;
 
@@ -429,9 +397,12 @@ int main(int argc,char **argv, char **envp)
       strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
       rows++;
       break;
+    case 'M':
+      strlcpy(cfg_cmdline[rows], "print_markers: true", SRVBUFLEN);
+      rows++;
+      break;
     case 'A':
-      strlcpy(cfg_cmdline[rows], "print_output_file_append: ", SRVBUFLEN);
-      strncat(cfg_cmdline[rows], optarg, CFG_LINE_LEN(cfg_cmdline[rows]));
+      strlcpy(cfg_cmdline[rows], "print_output_file_append: true", SRVBUFLEN);
       rows++;
       break;
     case 'E':
@@ -586,18 +557,6 @@ int main(int argc,char **argv, char **envp)
     }
   }
 
-  if (config.daemon) {
-    list = plugins_list;
-    while (list) {
-      if (!strcmp(list->type.string, "print") && !list->cfg.print_output_file)
-        printf("INFO ( %s/%s ): Daemonizing. Bye bye screen.\n", list->name, list->type.string);
-      list = list->next;
-    }
-    if (debug || config.debug)
-      printf("WARN ( %s/core ): debug is enabled; forking in background. Logging to standard error (stderr) will get lost.\n", config.name);
-    daemonize();
-  }
-
   initsetproctitle(argc, argv, envp);
   if (config.syslog) {
     logf = parse_log_facility(config.syslog);
@@ -609,8 +568,7 @@ int main(int argc,char **argv, char **envp)
     Log(LOG_INFO, "INFO ( %s/core ): Start logging ...\n", config.name);
   }
 
-  if (config.logfile)
-  {
+  if (config.logfile) {
     config.logfile_fd = open_output_file(config.logfile, "a", FALSE);
     list = plugins_list;
     while (list) {
@@ -619,6 +577,23 @@ int main(int argc,char **argv, char **envp)
     }
   }
 
+  if (config.daemon) {
+    list = plugins_list;
+    while (list) {
+      if (!strcmp(list->type.string, "print") && !list->cfg.print_output_file)
+	printf("INFO ( %s/%s ): Daemonizing. Bye bye screen.\n", list->name, list->type.string);
+      list = list->next;
+    }
+
+    if (!config.syslog && !config.logfile) {
+      if (debug || config.debug) {
+	printf("WARN ( %s/core ): debug is enabled; forking in background. Logging to standard error (stderr) will get lost.\n", config.name);
+      }
+    }
+
+    daemonize();
+  }
+
   if (config.proc_priority) {
     int ret;
 
@@ -690,7 +665,7 @@ int main(int argc,char **argv, char **envp)
 	list->cfg.what_to_count |= COUNT_IP_TOS;
 	list->cfg.what_to_count |= COUNT_IP_PROTO;
 	if (list->cfg.networks_file ||
-	   ((list->cfg.nfacctd_bgp || list->cfg.nfacctd_bmp) && list->cfg.nfacctd_as == NF_AS_BGP)) {
+	   ((list->cfg.bgp_daemon || list->cfg.bmp_daemon) && list->cfg.nfacctd_as == NF_AS_BGP)) {
 	  list->cfg.what_to_count |= COUNT_SRC_AS;
 	  list->cfg.what_to_count |= COUNT_DST_AS;
 	  list->cfg.what_to_count |= COUNT_PEER_DST_IP;
@@ -718,7 +693,7 @@ int main(int argc,char **argv, char **envp)
                                        COUNT_PEER_SRC_AS|COUNT_PEER_DST_AS|COUNT_PEER_SRC_IP|COUNT_SRC_STD_COMM|
 				       COUNT_SRC_EXT_COMM|COUNT_SRC_AS_PATH|COUNT_SRC_MED|COUNT_SRC_LOCAL_PREF|
 				       COUNT_MPLS_VPN_RD)) ||
-	    (list->cfg.what_to_count_2 & (COUNT_LRG_COMM|COUNT_SRC_LRG_COMM))) {
+	    (list->cfg.what_to_count_2 & (COUNT_LRG_COMM|COUNT_SRC_LRG_COMM|COUNT_SRC_ROA|COUNT_DST_ROA))) {
 	  Log(LOG_ERR, "ERROR ( %s/core ): 'src_as', 'dst_as' and 'peer_dst_ip' are currently the only BGP-related primitives supported within the 'nfprobe' plugin.\n", config.name);
 	  exit_gracefully(1);
 	}
@@ -756,12 +731,10 @@ int main(int argc,char **argv, char **envp)
 	  config.handle_flows = TRUE;
 	}
 #if defined (WITH_NDPI)
-	{ // XXX: some if condition here
-	  list->cfg.what_to_count_2 |= COUNT_NDPI_CLASS;
-	}
+	if (list->cfg.ndpi_num_roots) list->cfg.what_to_count_2 |= COUNT_NDPI_CLASS;
 #endif
         if (list->cfg.networks_file ||
-	   ((list->cfg.nfacctd_bgp || list->cfg.nfacctd_bmp) && list->cfg.nfacctd_as == NF_AS_BGP)) {
+	   ((list->cfg.bgp_daemon || list->cfg.bmp_daemon) && list->cfg.nfacctd_as == NF_AS_BGP)) {
           list->cfg.what_to_count |= COUNT_SRC_AS;
           list->cfg.what_to_count |= COUNT_DST_AS;
           list->cfg.what_to_count |= COUNT_PEER_DST_IP;
@@ -778,7 +751,7 @@ int main(int argc,char **argv, char **envp)
                                        COUNT_PEER_SRC_AS|COUNT_PEER_DST_AS|COUNT_PEER_SRC_IP|COUNT_SRC_STD_COMM|
 				       COUNT_SRC_EXT_COMM|COUNT_SRC_AS_PATH|COUNT_SRC_MED|COUNT_SRC_LOCAL_PREF|
 				       COUNT_MPLS_VPN_RD)) ||
-	    (list->cfg.what_to_count_2 & (COUNT_LRG_COMM|COUNT_SRC_LRG_COMM))) {
+	    (list->cfg.what_to_count_2 & (COUNT_LRG_COMM|COUNT_SRC_LRG_COMM|COUNT_SRC_ROA|COUNT_DST_ROA))) {
           Log(LOG_ERR, "ERROR ( %s/core ): 'src_as', 'dst_as' and 'peer_dst_ip' are currently the only BGP-related primitives supported within the 'sfprobe' plugin.\n", config.name);
           exit_gracefully(1);
         }
@@ -806,9 +779,13 @@ int main(int argc,char **argv, char **envp)
                         COUNT_MPLS_STACK_DEPTH))
           list->cfg.data_type |= PIPE_TYPE_MPLS;
 
-	if (list->cfg.what_to_count_2 & (COUNT_TUNNEL_SRC_HOST|COUNT_TUNNEL_DST_HOST|
-			COUNT_TUNNEL_IP_PROTO|COUNT_TUNNEL_IP_TOS))
+	if (list->cfg.what_to_count_2 & (COUNT_TUNNEL_SRC_MAC|COUNT_TUNNEL_DST_MAC|
+			COUNT_TUNNEL_SRC_HOST|COUNT_TUNNEL_DST_HOST|COUNT_TUNNEL_IP_PROTO|
+			COUNT_TUNNEL_IP_TOS|COUNT_TUNNEL_SRC_PORT|COUNT_TUNNEL_DST_PORT|
+			COUNT_VXLAN)) {
 	  list->cfg.data_type |= PIPE_TYPE_TUN;
+	  cb_data.has_tun_prims = TRUE;
+	}
 
         if (list->cfg.what_to_count_2 & (COUNT_LABEL))
           list->cfg.data_type |= PIPE_TYPE_VLEN;
@@ -848,10 +825,10 @@ int main(int argc,char **argv, char **envp)
           else {
             if ((list->cfg.nfacctd_net == NF_NET_NEW && !list->cfg.networks_file) ||
                 (list->cfg.nfacctd_net == NF_NET_STATIC && !list->cfg.networks_mask) ||
-                (list->cfg.nfacctd_net == NF_NET_BGP && !list->cfg.nfacctd_bgp && !list->cfg.nfacctd_bmp) ||
+                (list->cfg.nfacctd_net == NF_NET_BGP && !list->cfg.bgp_daemon && !list->cfg.bmp_daemon) ||
                 (list->cfg.nfacctd_net == NF_NET_IGP && !list->cfg.nfacctd_isis) ||
                 (list->cfg.nfacctd_net == NF_NET_KEEP)) {
-              Log(LOG_ERR, "ERROR ( %s/%s ): network aggregation selected but none of 'bgp_daemon', 'isis_daemon', 'networks_file', 'networks_mask' is specified. Exiting ...\n\n", list->name, list->type.string);
+              Log(LOG_ERR, "ERROR ( %s/%s ): network aggregation selected but none of 'bgp_daemon', 'bmp_daemon', 'isis_daemon', 'networks_file', 'networks_mask' is specified. Exiting ...\n\n", list->name, list->type.string);
               exit_gracefully(1);
             }
 	    if (list->cfg.nfacctd_net & NF_NET_FALLBACK && list->cfg.networks_file)
@@ -922,13 +899,13 @@ int main(int argc,char **argv, char **envp)
   load_networks(config.networks_file, &nt, &nc);
 
   if (config.pcap_interfaces_map) {
-    pcap_interfaces_map_initialize(&pcap_if_map);
-    pcap_interfaces_map_initialize(&bkp_pcap_if_map);
-    pcap_interfaces_map_load(&pcap_if_map);
+    pm_pcap_interfaces_map_initialize(&pm_pcap_if_map);
+    pm_pcap_interfaces_map_initialize(&pm_bkp_pcap_if_map);
+    pm_pcap_interfaces_map_load(&pm_pcap_if_map);
   }
   else {
-    pcap_if_map.list = NULL;
-    pcap_if_map.num = 0;
+    pm_pcap_if_map.list = NULL;
+    pm_pcap_if_map.num = 0;
   }
 
   if (!config.pcap_direction) config.pcap_direction = PCAP_D_INOUT;
@@ -936,13 +913,18 @@ int main(int argc,char **argv, char **envp)
   /* If any device/savefile have been specified, choose a suitable device
      where to listen for traffic */
   if (!config.pcap_if && !config.pcap_savefile && !config.pcap_interfaces_map) {
-    Log(LOG_WARNING, "WARN ( %s/core ): Selecting a suitable device.\n", config.name);
-    config.pcap_if = pcap_lookupdev(errbuf);
-    if (!config.pcap_if) {
-      Log(LOG_ERR, "ERROR ( %s/core ): Unable to find a suitable device. Exiting.\n", config.name);
+    char errbuf[PCAP_ERRBUF_SIZE];
+
+    Log(LOG_WARNING, "WARN ( %s/core ): Selecting a suitable devices.\n", config.name);
+
+    ret = pcap_findalldevs(&pm_pcap_ifs, errbuf);
+    if (ret == ERR || !pm_pcap_ifs) {
+      Log(LOG_ERR, "ERROR ( %s/core ): Unable to get interfaces list: %s. Exiting.\n", config.name, errbuf);
       exit_gracefully(1);
     }
-    else Log(LOG_DEBUG, "DEBUG ( %s/core ): device is %s\n", config.name, config.pcap_if);
+
+    config.pcap_if = pm_pcap_ifs[0].name; 
+    Log(LOG_DEBUG, "DEBUG ( %s/core ): device is %s\n", config.name, config.pcap_if);
   }
 
   /* reading filter; if it exists, we'll take an action later */
@@ -962,52 +944,72 @@ int main(int argc,char **argv, char **envp)
   FD_ZERO(&bkp_read_descs);
 
   if (config.pcap_if) {
-    ret = pm_pcap_add_interface(&device.list[0], config.pcap_if, NULL, psize);
+    ret = pm_pcap_add_interface(&devices.list[0], config.pcap_if, NULL, psize);
     if (!ret) {
-      cb_data.device = &device.list[0];
-      device.num = 1;
+      cb_data.device = &devices.list[0];
+      devices.num = 1;
     }
   }
   else if (config.pcap_interfaces_map) {
-    struct pcap_interface *pcap_if_entry;
-    int pcap_if_idx = 0;
+    struct pm_pcap_interface *pm_pcap_if_entry;
+    int pm_pcap_if_idx = 0;
     char *ifname;
 
-    while ((ifname = pcap_interfaces_map_getnext_ifname(&pcap_if_map, &pcap_if_idx))) {
-      if (device.num == PCAP_MAX_INTERFACES) {
+    while ((ifname = pm_pcap_interfaces_map_getnext_ifname(&pm_pcap_if_map, &pm_pcap_if_idx))) {
+      if (devices.num == PCAP_MAX_INTERFACES) {
 	Log(LOG_ERR, "ERROR ( %s/core ): Maximum number of interfaces reached (%u). Exiting.\n", config.name, PCAP_MAX_INTERFACES);
 	exit_gracefully(1);
       }
 
-      pcap_if_entry = pcap_interfaces_map_getentry_by_ifname(&pcap_if_map, ifname);
-      ret = pm_pcap_add_interface(&device.list[device.num], ifname, pcap_if_entry, psize);
+      pm_pcap_if_entry = pm_pcap_interfaces_map_getentry_by_ifname(&pm_pcap_if_map, ifname);
+      ret = pm_pcap_add_interface(&devices.list[devices.num], ifname, pm_pcap_if_entry, psize);
       if (!ret) {
-	if (bkp_select_fd <= device.list[device.num].fd) {
-	  bkp_select_fd = device.list[device.num].fd;
+	if (bkp_select_fd <= devices.list[devices.num].fd) {
+	  bkp_select_fd = devices.list[devices.num].fd;
 	  bkp_select_fd++;
 	}
 
-	if (device.list[device.num].fd) FD_SET(device.list[device.num].fd, &bkp_read_descs);
-	device.num++;
+	if (devices.list[devices.num].fd) FD_SET(devices.list[devices.num].fd, &bkp_read_descs);
+	devices.num++;
       }
     }
   }
   else if (config.pcap_savefile) {
-    open_pcap_savefile(&device.list[0], config.pcap_savefile);
-    pm_pcap_add_filter(&device.list[0]);
-    cb_data.device = &device.list[0];
-    device.num = 1;
-    pcap_savefile_round = 1;
+    open_pcap_savefile(&devices.list[0], config.pcap_savefile);
+    pm_pcap_add_filter(&devices.list[0]);
+    cb_data.device = &devices.list[0];
+    devices.num = 1;
+    pm_pcap_savefile_round = 1;
   }
 
   /* signal handling we want to inherit to plugins (when not re-defined elsewhere) */
-  signal(SIGCHLD, startup_handle_falling_child); /* takes note of plugins failed during startup phase */
-  signal(SIGHUP, reload); /* handles reopening of syslog channel */
-  signal(SIGUSR1, push_stats); /* logs various statistics via Log() calls */
-  signal(SIGUSR2, reload_maps); /* sets to true the reload_maps flag */
-  signal(SIGPIPE, SIG_IGN); /* we want to exit gracefully when a pipe is broken */
+  memset(&sighandler_action, 0, sizeof(sighandler_action)); /* To ensure the struct holds no garbage values */
+  sigemptyset(&sighandler_action.sa_mask);  /* Within a signal handler all the signals are enabled */
+  sighandler_action.sa_flags = SA_RESTART;  /* To enable re-entering a system call afer done with signal handling */
 
-  if (config.nfacctd_bgp && config.nfacctd_bmp) {
+  sighandler_action.sa_handler = startup_handle_falling_child;
+  sigaction(SIGCHLD, &sighandler_action, NULL);
+
+  /* handles reopening of syslog channel */
+  sighandler_action.sa_handler = reload;
+  sigaction(SIGHUP, &sighandler_action, NULL);
+
+  /* logs various statistics via Log() calls */
+  sighandler_action.sa_handler = push_stats;
+  sigaction(SIGUSR1, &sighandler_action, NULL);
+
+  /* sets to true the reload_maps flag */
+  sighandler_action.sa_handler = reload_maps;
+  sigaction(SIGUSR2, &sighandler_action, NULL);
+
+  /* we want to exit gracefully when a pipe is broken */
+  sighandler_action.sa_handler = SIG_IGN;
+  sigaction(SIGPIPE, &sighandler_action, NULL);
+
+  sighandler_action.sa_handler = PM_sigalrm_noop_handler;
+  sigaction(SIGALRM, &sighandler_action, NULL);
+
+  if (config.bgp_daemon && config.bmp_daemon) {
     Log(LOG_ERR, "ERROR ( %s/core ): bgp_daemon and bmp_daemon are currently mutual exclusive. Exiting.\n", config.name);
     exit_gracefully(1);
   }
@@ -1019,25 +1021,27 @@ int main(int argc,char **argv, char **envp)
     nfacctd_isis_wrapper();
 
     /* Let's give the ISIS thread some advantage to create its structures */
-    sleep(5);
+    sleep(DEFAULT_SLOTH_SLEEP_TIME);
   }
 
   /* starting the BGP thread */
-  if (config.nfacctd_bgp) {
+  if (config.bgp_daemon) {
+    int sleep_time = DEFAULT_SLOTH_SLEEP_TIME;
+
     req.bpf_filter = TRUE;
 
-    if (config.nfacctd_bgp_stdcomm_pattern_to_asn && config.nfacctd_bgp_lrgcomm_pattern_to_asn) {
+    if (config.bgp_daemon_stdcomm_pattern_to_asn && config.bgp_daemon_lrgcomm_pattern_to_asn) {
       Log(LOG_ERR, "ERROR ( %s/core ): bgp_stdcomm_pattern_to_asn and bgp_lrgcomm_pattern_to_asn are mutual exclusive. Exiting.\n", config.name);
       exit_gracefully(1);
     }
 
-    load_comm_patterns(&config.nfacctd_bgp_stdcomm_pattern, &config.nfacctd_bgp_extcomm_pattern,
-                        &config.nfacctd_bgp_lrgcomm_pattern, &config.nfacctd_bgp_stdcomm_pattern_to_asn,
-			&config.nfacctd_bgp_lrgcomm_pattern_to_asn);
+    load_comm_patterns(&config.bgp_daemon_stdcomm_pattern, &config.bgp_daemon_extcomm_pattern,
+                        &config.bgp_daemon_lrgcomm_pattern, &config.bgp_daemon_stdcomm_pattern_to_asn,
+			&config.bgp_daemon_lrgcomm_pattern_to_asn);
 
-    if (config.nfacctd_bgp_peer_as_src_type == BGP_SRC_PRIMITIVES_MAP) {
-      if (config.nfacctd_bgp_peer_as_src_map) {
-        load_id_file(MAP_BGP_PEER_AS_SRC, config.nfacctd_bgp_peer_as_src_map, &bpas_table, &req, &bpas_map_allocated);
+    if (config.bgp_daemon_peer_as_src_type == BGP_SRC_PRIMITIVES_MAP) {
+      if (config.bgp_daemon_peer_as_src_map) {
+        load_id_file(MAP_BGP_PEER_AS_SRC, config.bgp_daemon_peer_as_src_map, &bpas_table, &req, &bpas_map_allocated);
 	cb_data.bpas_table = (u_char *) &bpas_table;
       }
       else {
@@ -1047,9 +1051,9 @@ int main(int argc,char **argv, char **envp)
     }
     else cb_data.bpas_table = NULL;
 
-    if (config.nfacctd_bgp_src_local_pref_type == BGP_SRC_PRIMITIVES_MAP) {
-      if (config.nfacctd_bgp_src_local_pref_map) {
-        load_id_file(MAP_BGP_SRC_LOCAL_PREF, config.nfacctd_bgp_src_local_pref_map, &blp_table, &req, &blp_map_allocated);
+    if (config.bgp_daemon_src_local_pref_type == BGP_SRC_PRIMITIVES_MAP) {
+      if (config.bgp_daemon_src_local_pref_map) {
+        load_id_file(MAP_BGP_SRC_LOCAL_PREF, config.bgp_daemon_src_local_pref_map, &blp_table, &req, &blp_map_allocated);
         cb_data.blp_table = (u_char *) &blp_table;
       }
       else {
@@ -1059,9 +1063,9 @@ int main(int argc,char **argv, char **envp)
     }
     else cb_data.blp_table = NULL;
 
-    if (config.nfacctd_bgp_src_med_type == BGP_SRC_PRIMITIVES_MAP) {
-      if (config.nfacctd_bgp_src_med_map) {
-        load_id_file(MAP_BGP_SRC_MED, config.nfacctd_bgp_src_med_map, &bmed_table, &req, &bmed_map_allocated);
+    if (config.bgp_daemon_src_med_type == BGP_SRC_PRIMITIVES_MAP) {
+      if (config.bgp_daemon_src_med_map) {
+        load_id_file(MAP_BGP_SRC_MED, config.bgp_daemon_src_med_map, &bmed_table, &req, &bmed_map_allocated);
         cb_data.bmed_table = (u_char *) &bmed_table;
       }
       else {
@@ -1071,8 +1075,8 @@ int main(int argc,char **argv, char **envp)
     }
     else cb_data.bmed_table = NULL;
 
-    if (config.nfacctd_bgp_to_agent_map) {
-      load_id_file(MAP_BGP_TO_XFLOW_AGENT, config.nfacctd_bgp_to_agent_map, &bta_table, &req, &bta_map_allocated);
+    if (config.bgp_daemon_to_xflow_agent_map) {
+      load_id_file(MAP_BGP_TO_XFLOW_AGENT, config.bgp_daemon_to_xflow_agent_map, &bta_table, &req, &bta_map_allocated);
       cb_data.bta_table = (u_char *) &bta_table;
     }
     else {
@@ -1083,13 +1087,27 @@ int main(int argc,char **argv, char **envp)
     /* Limiting BGP peers to only two: one would suffice in pmacctd
        but in case maps are reloadable (ie. bta), it could be handy
        to keep a backup feed in memory */
-    config.nfacctd_bgp_max_peers = 2;
+    config.bgp_daemon_max_peers = 2;
 
-    cb_data.f_agent = (char *)&client;
-    nfacctd_bgp_wrapper();
+    cb_data.f_agent = (u_char *) &client;
+    bgp_daemon_wrapper();
 
     /* Let's give the BGP thread some advantage to create its structures */
-    sleep(5);
+    if (config.rpki_roas_file || config.rpki_rtr_cache) sleep_time += DEFAULT_SLOTH_SLEEP_TIME;
+    sleep(sleep_time);
+  }
+
+  /* starting the BMP thread */
+  if (config.bmp_daemon) {
+    int sleep_time = DEFAULT_SLOTH_SLEEP_TIME;
+
+    req.bpf_filter = TRUE;
+
+    bmp_daemon_wrapper();
+
+    /* Let's give the BMP thread some advantage to create its structures */
+    if (config.rpki_roas_file || config.rpki_rtr_cache) sleep_time += DEFAULT_SLOTH_SLEEP_TIME;
+    sleep(sleep_time);
   }
 
 #if defined WITH_GEOIP
@@ -1119,9 +1137,15 @@ int main(int argc,char **argv, char **envp)
 
   /* signals to be handled only by the core process;
      we set proper handlers after plugin creation */
-  signal(SIGINT, my_sigint_handler);
-  signal(SIGTERM, my_sigint_handler);
-  signal(SIGCHLD, handle_falling_child);
+  sighandler_action.sa_handler = PM_sigint_handler;
+  sigaction(SIGINT, &sighandler_action, NULL);
+
+  sighandler_action.sa_handler = PM_sigint_handler;
+  sigaction(SIGTERM, &sighandler_action, NULL);
+
+  sighandler_action.sa_handler = handle_falling_child;
+  sigaction(SIGCHLD, &sighandler_action, NULL);
+
   kill(getpid(), SIGCHLD);
 
   /* When reading packets from a savefile, things are lightning fast; we will sit
@@ -1134,29 +1158,49 @@ int main(int argc,char **argv, char **envp)
     else sleep(config.pcap_sf_delay);
   }
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_core_handler);
+  }
+#endif
+
+  sigemptyset(&cb_data.sig.set);
+  sigaddset(&cb_data.sig.set, SIGCHLD);
+  sigaddset(&cb_data.sig.set, SIGHUP);
+  sigaddset(&cb_data.sig.set, SIGUSR1);
+  sigaddset(&cb_data.sig.set, SIGUSR2);
+  sigaddset(&cb_data.sig.set, SIGTERM);
+  if (config.daemon) {
+    sigaddset(&cb_data.sig.set, SIGINT);
+  }
+  cb_data.sig.is_set = TRUE;
+
   /* Main loop (for the case of a single interface): if pcap_loop() exits
      maybe an error occurred; we will try closing and reopening again our
      listening device */
   if (!config.pcap_interfaces_map) {
     for (;;) {
-      if (!device.list[0].active) {
+      if (!devices.list[0].active) {
 	Log(LOG_WARNING, "WARN ( %s/core ): [%s] has become unavailable; throttling ...\n", config.name, config.pcap_if);
-	ret = pm_pcap_add_interface(&device.list[0], config.pcap_if, NULL, psize);
+	ret = pm_pcap_add_interface(&devices.list[0], config.pcap_if, NULL, psize);
 	if (!ret) {
-	  cb_data.device = &device.list[0];
-	  device.num = 1;
+	  cb_data.device = &devices.list[0];
+	  devices.num = 1;
 	}
       }
 
       read_packet:
-      pcap_loop(device.list[0].dev_desc, -1, pcap_cb, (u_char *) &cb_data);
-      pcap_close(device.list[0].dev_desc);
+      pcap_loop(devices.list[0].dev_desc, -1, pm_pcap_cb, (u_char *) &cb_data);
+      pcap_close(devices.list[0].dev_desc);
 
       if (config.pcap_savefile) {
 	if (config.pcap_sf_replay < 0 ||
-	    (config.pcap_sf_replay > 0 && pcap_savefile_round < config.pcap_sf_replay)) {
-	  pcap_savefile_round++;
-	  open_pcap_savefile(&device.list[0], config.pcap_savefile);
+	    (config.pcap_sf_replay > 0 && pm_pcap_savefile_round < config.pcap_sf_replay)) {
+	  pm_pcap_savefile_round++;
+	  open_pcap_savefile(&devices.list[0], config.pcap_savefile);
 	  if (config.pcap_sf_delay) sleep(config.pcap_sf_delay);
 
 	  goto read_packet;
@@ -1169,7 +1213,7 @@ int main(int argc,char **argv, char **envp)
         }
         stop_all_childs();
       }
-      device.list[0].active = FALSE;
+      devices.list[0].active = FALSE;
     }
   }
   else {
@@ -1177,66 +1221,66 @@ int main(int argc,char **argv, char **envp)
       select_fd = bkp_select_fd;
       memcpy(&read_descs, &bkp_read_descs, sizeof(bkp_read_descs));
 
-      select_num = select(select_fd, &read_descs, NULL, NULL, NULL);
+      select(select_fd, &read_descs, NULL, NULL, NULL);
 
       if (reload_map_pmacctd) {
-	struct pcap_interface *pcap_if_entry;
-	int pcap_if_idx = 0;
+	struct pm_pcap_interface *pm_pcap_if_entry;
+	int pm_pcap_if_idx = 0;
 	char *ifname;
 
-	pcap_interfaces_map_copy(&bkp_pcap_if_map, &pcap_if_map);
-	pcap_interfaces_map_destroy(&pcap_if_map);
-	pcap_interfaces_map_load(&pcap_if_map);
+	pm_pcap_interfaces_map_copy(&pm_bkp_pcap_if_map, &pm_pcap_if_map);
+	pm_pcap_interfaces_map_destroy(&pm_pcap_if_map);
+	pm_pcap_interfaces_map_load(&pm_pcap_if_map);
 
-	pm_pcap_device_copy_all(&bkp_device, &device);
-	pm_pcap_device_initialize(&device);
+	pm_pcap_device_copy_all(&bkp_devices, &devices);
+	pm_pcap_device_initialize(&devices);
 
 	/* Add interfaces and re-build relevant structs */
-	while ((ifname = pcap_interfaces_map_getnext_ifname(&pcap_if_map, &pcap_if_idx))) {
-	  if (!pcap_interfaces_map_lookup_ifname(&bkp_pcap_if_map, ifname)) {
-	    if (device.num == PCAP_MAX_INTERFACES) {
+	while ((ifname = pm_pcap_interfaces_map_getnext_ifname(&pm_pcap_if_map, &pm_pcap_if_idx))) {
+	  if (!pm_pcap_interfaces_map_lookup_ifname(&pm_bkp_pcap_if_map, ifname)) {
+	    if (devices.num == PCAP_MAX_INTERFACES) {
 	      Log(LOG_WARNING, "WARN ( %s/core ): Maximum number of interfaces reached (%u). Ignoring '%s'.\n", config.name, PCAP_MAX_INTERFACES, ifname);
 	    }
 	    else {
-	      pcap_if_entry = pcap_interfaces_map_getentry_by_ifname(&pcap_if_map, ifname);
-	      if (!pm_pcap_add_interface(&device.list[device.num], ifname, pcap_if_entry, psize)) {
-		if (bkp_select_fd <= device.list[device.num].fd) {
-		  bkp_select_fd = device.list[device.num].fd;
+	      pm_pcap_if_entry = pm_pcap_interfaces_map_getentry_by_ifname(&pm_pcap_if_map, ifname);
+	      if (!pm_pcap_add_interface(&devices.list[devices.num], ifname, pm_pcap_if_entry, psize)) {
+		if (bkp_select_fd <= devices.list[devices.num].fd) {
+		  bkp_select_fd = devices.list[devices.num].fd;
 		  bkp_select_fd++;
 		}
 
-		if (device.list[device.num].fd && !FD_ISSET(device.list[device.num].fd, &bkp_read_descs)) {
-		  FD_SET(device.list[device.num].fd, &bkp_read_descs);
+		if (devices.list[devices.num].fd && !FD_ISSET(devices.list[devices.num].fd, &bkp_read_descs)) {
+		  FD_SET(devices.list[devices.num].fd, &bkp_read_descs);
 		}
 	
-		device.num++;
+		devices.num++;
 	      }
 	    }
 	  }
           else {
 	    int device_idx;
 
-	    device_idx = pm_pcap_device_getindex_byifname(&bkp_device, ifname);
+	    device_idx = pm_pcap_device_getindex_byifname(&bkp_devices, ifname);
 	    if (device_idx >= 0) {
-	      Log(LOG_INFO, "INFO ( %s/core ): [%s,%u] link type is: %d\n", config.name, bkp_device.list[device_idx].str,
-		  bkp_device.list[device_idx].id, bkp_device.list[device_idx].link_type);
-	      pm_pcap_device_copy_entry(&device, &bkp_device, device_idx);
+	      Log(LOG_INFO, "INFO ( %s/core ): [%s,%u] link type is: %d\n", config.name, bkp_devices.list[device_idx].str,
+		  bkp_devices.list[device_idx].id, bkp_devices.list[device_idx].link_type);
+	      pm_pcap_device_copy_entry(&devices, &bkp_devices, device_idx);
 	    }
 	    else Log(LOG_WARNING, "WARN ( %s/core ): Mayday. Interface '%s' went lost.\n", config.name, ifname);
 	  }
 	}
 
 	/* Remove unlisted interfaces */
-	pcap_if_idx = 0;
-	while ((ifname = pcap_interfaces_map_getnext_ifname(&bkp_pcap_if_map, &pcap_if_idx))) {
-	  if (!pcap_interfaces_map_lookup_ifname(&pcap_if_map, ifname)) {
+	pm_pcap_if_idx = 0;
+	while ((ifname = pm_pcap_interfaces_map_getnext_ifname(&pm_bkp_pcap_if_map, &pm_pcap_if_idx))) {
+	  if (!pm_pcap_interfaces_map_lookup_ifname(&pm_pcap_if_map, ifname)) {
             int device_idx;
           
-	    device_idx = pm_pcap_device_getindex_byifname(&bkp_device, ifname);
+	    device_idx = pm_pcap_device_getindex_byifname(&bkp_devices, ifname);
 	    if (device_idx >= 0) {
-	      Log(LOG_INFO, "INFO ( %s/core ): [%s,%u] removed.\n", config.name, bkp_device.list[device_idx].str, bkp_device.list[device_idx].id);
-	      FD_CLR(bkp_device.list[device_idx].fd, &bkp_read_descs);
-	      pcap_close(bkp_device.list[device_idx].dev_desc);
+	      Log(LOG_INFO, "INFO ( %s/core ): [%s,%u] removed.\n", config.name, bkp_devices.list[device_idx].str, bkp_devices.list[device_idx].id);
+	      FD_CLR(bkp_devices.list[device_idx].fd, &bkp_read_descs);
+	      pcap_close(bkp_devices.list[device_idx].dev_desc);
             }
 	    else Log(LOG_WARNING, "WARN ( %s/core ): Mayday. Interface '%s' went lost (2).\n", config.name, ifname);
 	  }
@@ -1245,12 +1289,12 @@ int main(int argc,char **argv, char **envp)
 	reload_map_pmacctd = FALSE;
       }
 
-      for (dev_ptr = NULL, index = 0; index < device.num; index++) {
-        int loc_idx = (index + index_rr) % device.num;
+      for (dev_ptr = NULL, index = 0; index < devices.num; index++) {
+        int loc_idx = (index + index_rr) % devices.num;
 
-	if (device.list[loc_idx].fd && FD_ISSET(device.list[loc_idx].fd, &read_descs)) {
-	  dev_ptr = &device.list[loc_idx];
-          index_rr = (index_rr + 1) % device.num;
+	if (devices.list[loc_idx].fd && FD_ISSET(devices.list[loc_idx].fd, &read_descs)) {
+	  dev_ptr = &devices.list[loc_idx];
+          index_rr = (index_rr + 1) % devices.num;
           break;
 	}
       }
@@ -1259,7 +1303,7 @@ int main(int argc,char **argv, char **envp)
 	pkt_body = pcap_next(dev_ptr->dev_desc, &pkt_hdr); 
 	if (pkt_body) {
 	  cb_data.device = dev_ptr;
-	  pcap_cb((u_char *) &cb_data, &pkt_hdr, pkt_body);
+	  pm_pcap_cb((u_char *) &cb_data, &pkt_hdr, pkt_body);
 	}
       }
     }
